#include <stdio.h>
#include "manager.h"
#include "mouinter.h"
#include "object.h"
#include "depend.h"
#include "actids.h"

#ifdef __MSDOS__
#include <dos.h>
#endif

#ifdef KBHIT
#include <conio.h>
#endif

//////////////////////
//                  //
// ObjMgr functions //
//                  //
//////////////////////

//
// Initialize static member of ObjMgr
//
InterCs* ObjMgr::InterCList = 0;

/////////////////////////////////////////////////////
//                                                 //
// Private Function                                //
// This function does not deallocate the object    //
// it only remove the object from the list         //
//                                                 //
/////////////////////////////////////////////////////
BBOOL _pascal_
ObjList::UnRegObj (Object *obj )
{
    Object *tmpObjPtr, *prevObjPtr;

    tmpObjPtr = ObjHead;
    prevObjPtr = 0;
    
    if (obj == 0 ) return FALSE;       // Make sure it's not a dummy pointer
    if (obj == Current ) Current = 0;  // Can't directly unreg current
                                       // because need to ensure the object
                                       // before current is not lost.
    while (tmpObjPtr ) {
        if (tmpObjPtr == obj ) {       // found
            if (!tmpObjPtr->next )     // Tail or only one object

                if (!prevObjPtr )       // check if only one
                    ObjTail = ObjHead = 0;  // only one => all gone
                else {
                    prevObjPtr->next = 0;      // it's tail! take it out.
                    ObjTail = prevObjPtr;
                }

            else if (prevObjPtr )              // in Middle?
                prevObjPtr->next = tmpObjPtr->next;   // take it out
            else                               // else it's head
                ObjHead = tmpObjPtr->next;     // take out head

            TotalObj--;
            return TRUE;
        }
		prevObjPtr = tmpObjPtr;
		tmpObjPtr = tmpObjPtr->next;
	}
	return FALSE;
} // end of private function UnRegObj

//
// Constructors
//
_pascal_ ObjList::ObjList( Object *object )
{
    if ( object )
        TotalObj = 1;
    else
        TotalObj = 0;
    ObjTail = ObjHead = Current = object;
}

_pascal_ ObjList::~ObjList() {}

_pascal_ ObjMgr :: ObjMgr(Object *object )
        : ObjList( object )
{
	stop     = FALSE;
	PrivateInterCList = 0;
}

_pascal_ ObjMgr::~ObjMgr()
{
    InterCs *tmp;
    while( PrivateInterCList ) {
        tmp = PrivateInterCList;
        PrivateInterCList = PrivateInterCList->next;
        CUnRegInt( tmp->InterC, TRUE );
        delete tmp;
    }
}

ObjList& _pascal_ ObjList::operator= ( const ObjList &y )
{
    if ( this != &y ) {
        (*this).TotalObj = y.TotalObj;
        (*this).ObjHead = y.ObjHead;
        (*this).ObjTail = y.ObjTail;
        (*this).Current = y.Current;
    }
    return *this;
}

ObjMgr& _pascal_ ObjMgr::operator= (const ObjMgr &y)
{
    if (this != &y ) {
		(*this).stop = y.stop;
        (*this).InterCList = y.InterCList;
        (*this).PrivateInterCList = y.InterCList;
		ObjList::operator=( y );
    }
    return *this;
}

Object* _pascal_
ObjList::CGetObject (WORD id )
{
    Object *tmpObjPtr;
    tmpObjPtr = ObjHead;
    
    while (tmpObjPtr ) { 
        if (tmpObjPtr->ObjID == id )   // found
            return tmpObjPtr;
        tmpObjPtr = tmpObjPtr->next;
    }
    return 0;
}


BBOOL _pascal_
ObjList::CRegObject (Object *obj )
{
    Object *tmpPtr;
    tmpPtr = ObjHead;

    while ( tmpPtr ) {
        if ( tmpPtr == obj || tmpPtr->ObjID == obj->ObjID )
            return FALSE;
        tmpPtr = tmpPtr->next;
    }

    if (!ObjHead )
        ObjHead = ObjTail = obj;
    else { 
        ObjTail->next = obj;
        ObjTail = obj;
    }
    obj->next = 0;

    TotalObj ++;
    return TRUE;
}


//////////////////////////////////////////////////////////////
//                                                          //
// CUnRegObjByPtr( Object* object )                         //
// Unregister an object by passing in its pointer           //
//                                                          //
//////////////////////////////////////////////////////////////
BBOOL _pascal_
ObjList::CUnRegObjByPtr (Object *object )
{
    return UnRegObj( object );
}

//////////////////////////////////////////////////////////////
//                                                          //
// CUnRegObjByID( WORD id )                                  //
// Unregister an object by passing in its id                //
//                                                          //
//////////////////////////////////////////////////////////////
BBOOL _pascal_
ObjList::CUnRegObjByID (WORD id )
{
    return UnRegObj( CGetObject(id) );
}

//////////////////////////////////////////////////////////////
//                                                          //
// CRegInt( INTC regInt  )                                  //
//                                                          //
//////////////////////////////////////////////////////////////
void _pascal_
ObjMgr::CRegInt ( INTC regInt, Object *ObjInt, BYTE PrivateList )
{
    InterCs *tmpIntPtr, *ListHead;

    ListHead = InterCList;
    if (PrivateList) ListHead = PrivateInterCList;

    if ( ListHead ) {
        // add to tail
        tmpIntPtr = ListHead;
        
        while (tmpIntPtr->next )                         // find tail
            tmpIntPtr = tmpIntPtr->next;
        tmpIntPtr->next = new InterCs (regInt, ObjInt ); // add to list
    } else {
        ListHead = new InterCs (regInt, ObjInt ); // or it's first one
        if ( PrivateList ) PrivateInterCList = ListHead;
        else               InterCList = ListHead;
    }
}

BBOOL _pascal_
ObjMgr::CUnRegInt ( INTC unRegInt, BBOOL PrivateList )
{
    InterCs *tmpIntPtr, *prevIntPtr, *ListHead;
    BBOOL Success = FALSE;

    ListHead = InterCList;
    if (PrivateList) ListHead = PrivateInterCList;
    tmpIntPtr = ListHead;
    prevIntPtr = 0;
    
    while (tmpIntPtr ) { 
        if (tmpIntPtr->InterC == unRegInt ) {  // found
            if (!tmpIntPtr->next )     // Tail or Only one interceptor
                if (prevIntPtr )       // check if only one
                    prevIntPtr->next = 0;
            else                       // only one => all gone
                ListHead = 0;
            else if (prevIntPtr )      // middle
                prevIntPtr->next = tmpIntPtr->next;    // take it out
            else                                       // else it's head
                ListHead = tmpIntPtr->next;          // take out head
            
            delete tmpIntPtr;          // and delete it (middle or head)
            Success = TRUE;
            break;
        }
        prevIntPtr = tmpIntPtr;
        tmpIntPtr = tmpIntPtr->next;
    }
    if ( PrivateList ) PrivateInterCList = ListHead;
    else               InterCList = ListHead;
    return Success;
}

int _pascal_
ObjMgr::CallIntCs( InterCs *headPtr )
{
    static InterCs *BackUpPtr;
    static int CallResult;

    while( headPtr ) {
        CallResult = headPtr->InterC(headPtr->ObjIntCPtr);

        BackUpPtr = headPtr;
		headPtr = headPtr->next;
        if (CallResult ) {
            switch (CallResult ) {
                case UNREGME:
                    CUnRegInt (BackUpPtr->InterC );
                    break;
                case UNREGMEnSTOP:
                    CUnRegInt (BackUpPtr->InterC );
                    stop = 1;
                    break;
                case STOPNOW:
                    stop = 1;
                    return 0;
            }
        }
	}
    return 1;
}

int _pascal_
ObjMgr::CMgrLoop ( void )
{
    ObjMgr   *oldMgr;   // Since we are going to take over,
                        // better save the old guy

    stop = 0;
    oldMgr = CurrentMgr;
    CurrentMgr = this;

    while ( !stop ) {
    	#ifdef __LINUX__
    	mouse_update();
    	#endif

        if ( CallIntCs( InterCList ) )
            CallIntCs( PrivateInterCList );
        ObjHandler ();
    }
    CurrentMgr = oldMgr;
    return 0;
}

/////////////////////////////
// End of ObjMgr functions //
/////////////////////////////
