#ifdef VID_INTERFACE

#include "macros.h"
#define TURBOC_GRAPH
#define _640x480
#include "modeinfo.h"
#include <graphics.h>


//
// Some general Functions
//
#define v_SetMode               tc_SetMode(MODE)
#define v_TextMode              tc_TextMode()

// Draw a rectangle and fill it with color 'color'
#define v_RectFill( x1, y1, x2, y2, color )\
        __RectFill( x1, y1, x2, y2, color )

// Samething as above, but define this one for objects(just in case of changes)
#define v_ObjectFaceFill( x1, y1, x2, y2, color )\
        __RectFill( x1, y1, x2, y2, color )

void _pascal_ __RectFill( int, int, int, int, int );

// Define the whole palette ( all colors )
#define v_PutPal( buf )

// Draw one single pixel on screen at pos(x,y)
#define v_PutPixel(x, y, color ) putpixel( x, y, color )


// Draw a line from pos(x1,y1) to pos(x2,y2)
#define v_PutLine(x1, y1, x2, y2, color )  \
        _Line( x1, y1, x2, y2, color )
// Draw a XOR line
#define v_XPutLine( x1, y1, x2, y2, color )\
        _XLine( x1, y1, x2, y2, color )

void _pascal_ _XLine( int, int, int, int, int );
void _pascal_ _Line( int, int, int, int, int );

//
// Bitmap functions
//
// Grab a piece of bitmap from screen
WORD _pascal_ v_BMPsize( int, int );
#define v_Getbmp( x1, y1, width, height, buf ) \
        getimage( x1, y1, x1+width, y1+height, (void far*)buf )

// Put a piece of bitmap to screen
#define v_Putbmp( x1, y1, buf ) \
        putimage( x1, y1, (void far*) buf, COPY_PUT )


//
// User font informations
//
//#define FONT_AVAILABLE
#define FONTDIR "..\\tool\\"
#define USERFONTNAME "SMALTHIN.FNT"
#define CHARS 256
#define CHARH 8


//
// Initialize and deinitialize the video/mouse
//
int  tcgraph_Init(void);
void tcgraph_DeInit(void);

// Macro to get into the graphic mode( all applications will use this )

#define _InitVideo				tcgraph_Init()


// All application should use _RestoreVideo, which restore the screen to text
// mode.  However, if AUTO_DE_INIT is defined, restorevideo is not neccessary
// to be call at the end of applications.

//#define AUTO_DE_INIT

#ifndef AUTO_DE_INIT
#define _RestoreVideo           tcgraph_DeInit()
#else
#define _RestoreVideo
#endif


//
// Some useful routines
//
// Draw a rectangle

void _pascal_ __Rectangle(  int x1, int y1, int x2, int y2, int color );
void _pascal_  __XRectangle( int, int, int, int, int );

#define v_Rectangle( x1, y1, x2, y2, color ) \
        __Rectangle( x1, y1, x2, y2, color )
#define v_XRectangle( x1, y1, x2, y2, color ) \
        __XRectangle( x1, y1, x2, y2, color )

//
// Routines/Macros to display and manipulate fonts
//

// Current active font height
#define v_CharHeight() textheight("M")

// Return width of a character
#define v_CharWidth( c )      8


// Calculate horizontal pixels a text string is going to take on screen

int _pascal_ v_StrPixelLen( char *str );


// Print a text message, ignore mouse
// (x,y) (upper left pixel pos of the text on screen)
// fc - foreground color( text color )
// bc - background color
// len - Lenth of the string in pixels
// str - duh!
// returns: the pixel coordinate where the end of string is.

int _pascal_ __PrintMsg(int x, int y, int fc, int bc, int len, char *str);


// Print a text message, mouse aware
// See above function for descriptions of the parameters
// returns: the pixel coordinate where the end of string is.

int _pascal_ __PrintMsgMA(int x, int y, int fc, int bc, int len, char *str);


// This function is use to print string with pixel length longer than
// the number pass to parameter len.  It will not print charaters outside
// beyond the pixel length passed in allowed.
// len - length in pixels to print the string up to.
// See above function for descriptions of the other parameters
// returns: the pixel coordinate where the end of string is.

int _pascal_
	v_LimitPixelPrint(int x, int y, int fc, int bc, int len, char *str );

// This function will return the pixel length of a string
// str - duh!
// slen - count of charaters in the string(strlen) or whatever string length
//        you want to limit the function to.
// plen - pixel length to check for.  When the number of characters from the
//        the string have a total pixel length longer than plen, the function
//        stops and return that pixel length.

int _pascal_ v_StrPixelLenUpTo( char *str, int slen, int plen );


// This function takes a segment of a string defined by two string
// pointers, begin and end, and find a subsegment within it.
// To do so, it needs one more parameter, len, which is the pixel
// length of the subsegment.  The subsegment will be a pointer equals to
// (end - strlen equivalent to pixel length len).  This new pointer
// will then be return to the caller, along with the string position
// in the pass in by reference integer newPos.
//
// begin - define the begin of the segment
// end - define the end of the segment
// len - limit pixel length
// newPos - charater position of the return pointer from pointer begin.
//          i.e. begin + newPos = return pointer
//          or   end - strlen==len = return pointer
// example:
//
//     [...some charaters.|..........]
//     ^                  ^          ^
//     begin              |        end
//                  return pointer   |
//                        ^----+-----^
//                        |    |
//                        |  PixelLen == len
//                        |
//                        newPos(StrLen)
//
char* _pascal_ v_StrToFitPixels( char *begin, char *end, int len, int &newPos);

int _pascal_ v_CenterMsgY( int y1, int y2 );
int _pascal_ v_CenterMsgX( int x1, int x2, char *msg );

// macros refer to functions above, recommended.
// ShowMsg macros ignore mouse cursor
// PrintMsg macros hide mouse cursor before printing, but do not restore it.

#define v_PrintMsgLen( x, y, len, fc, bc, s ) \
		__PrintMsgMA( x, y, fc, bc, len, s )

#define v_ShowMsgLen( x, y, len, fc, bc, s ) \
		__PrintMsg( x, y, fc, bc, len, s )

#define v_PrintMsg( x, y, fc, bc, s ) \
		__PrintMsgMA( x, y, fc, bc, v_StrPixelLen( s ), (char*)s )

#define v_ShowMsg( x, y, fc, bc, s ) \
		__PrintMsg( x, y, fc, bc, v_StrPixelLen( s ), (char*)s )

////// VID_INTERFACE
#endif
//////
