#include <stdlib.h>
#include <dir.h>
#include <process.h>
#include "vidinter.h"  // Video interface
#include "timer.h"     // Interrupt driven timer
#include "fileutil.h"    // Read and write files
#include "depend.h"    // OS dependent stuff
#include "button.h"    // Push buttons
#include "viewmgr.h"   // View manager
#include "actids.h"	   // Action ids
#include "dialog.h"
#include "strbox.h"
#include "chkbox.h"
#include "keys.h"
#include "filedlg.id"
#include "filedlg.h"
#include "dir.h"
#include "buffer.h"
#include <ctype.h>

#ifdef __MSDOS__
#include <dos.h>       // Needed for msdos
#include <alloc.h>
#include <conio.h>
#endif

extern Timer friction;

char ExportFileName[MAXFILE+MAXEXT] = {"\0"};

static FileName fns[512];
static int FileIndex, TotalFiles;
static StrBox *FNBox;
static Dialog *dlg = 0;
static char ActionString[25];
static char WildCard[MAXFILE+MAXEXT];

#ifndef BUFFER
#error Need a buffer of at least 100 bytes
#endif

static void _pascal_ RedrawEntries( void )
{
    for ( int i = FILE_ENTRY1_ID; i <= FILE_ENTRY10_ID; i ++ )
        dlg->CSendMsg( dlg->CGetObject(i), REDRAW );
    dlg->CSendMsg( dlg->CGetObject(SCROLL_BAR_ID), REDRAW );
}

static int SetPath( char *path )
{
    getcwd( buffer, MAXPATH );
    if ( path && strcmp( ".", path ) != 0 ) {
        if ( buffer[strlen(buffer)-1] != '\\' )
            strcat( buffer, "\\" );
        strcat( buffer, path );
    }
    if ( chdir( buffer ) != 0 ) {
        sprintf( buffer, "Change to dir. '%s' failed.", buffer );
        v_PrintMsg( 0, 0, TextColor, FaceColor, buffer );
        return 0;
    }
    FileIndex = 0;
    TotalFiles = GetFiles( WildCard, fns );
    RedrawEntries();

    return 1;
}

#define _I_HAVE_MY_OWN_HANDLERS_
//
// Object ID Name: FILE_ENTRY1_ID
//
// Object ID Name: FILE_ENTRY2_ID
//
// Object ID Name: FILE_ENTRY3_ID
//
// Object ID Name: FILE_ENTRY4_ID
//
// Object ID Name: FILE_ENTRY5_ID
//
// Object ID Name: FILE_ENTRY6_ID
//
// Object ID Name: FILE_ENTRY7_ID
//
// Object ID Name: FILE_ENTRY8_ID
//
// Object ID Name: FILE_ENTRY9_ID
//
// Object ID Name: FILE_ENTRY10_ID
//
DeclareObjH( FileEntry )
{
    ScrObj &t = *((ScrObj*)msg->ToObj);
    // entry1 to entry10 must have ids xx1-x10
    int i = t.CGetID() - FILE_ENTRY1_ID + FileIndex;
    switch( msg->actId ) {
        case REDRAW:
            if ( t.CMouseOnMe())
                m_HideMouse;
            t.CClrContext( FaceColor );
            if ( i < TotalFiles ) {
                v_PrintMsg( t.x1() + 2,
                            v_CenterMsgY( t.y1(), t.y2()),
                            TextColor, FaceColor, fns[i].CName() );
            } else
                m_ShowMouse;

            break;
        default:
            break;
    }
    if ( msg->mouButton == LCLICK && i < TotalFiles ) {
        if ( fns[i].CAtt() & 0x10 ) {
            fns[i].CName()[strlen(fns[i].CName())-1] = 0;
            SetPath( fns[i].CName() );
        } else {
            strcpy( ExportFileName, fns[i].CName() );
            FNBox->CSetName( ExportFileName );
            dlg->CSendMsg( FNBox, REDRAW );
        }
    }
    return 0;
}

int ChangeDrive( void )
{
    char *path = (char*)FNBox->CStr();
    if ( path[strlen(path)-1] == ':' ) {
        int i = (int)(toupper(path[0]) - 'A' );
        if ( i >= 0 )
            if ( setdisk( i ) != 0 )
                SetPath( 0 );
            else {
                sprintf( buffer, "Change to driver: %d %s failed.", i, path );
                v_PrintMsg( 0, 0, TextColor, FaceColor, buffer );
            }
        return 1;
    }
    return 0;
}

//
// Object ID Name: SCROLL_BAR_ID
//
DeclareObjH( ScrollBar )
{
    static int size = 0, y = 0, x1, x2, yi, oyi = -1, y1;
    static int my = 0, myOffSet = 0;
    static BBOOL Scrolling = FALSE;
    Button &t = *((Button*)msg->ToObj);
    if ( !y ) {
        y = t.y2() - t.y1() - 1;
        x1 = t.x1() + 1;
        x2 = t.x2() - 1;
        y1 = t.y1() + 1;
    }
    if ( msg->actId == REDRAW ) {
        m_HideMouse;
        yi = (FileIndex*y)/TotalFiles + y1;
        if ( oyi > -1 ) {
            v_Rectangle( x1, oyi, x2, oyi + size, FaceColor );
        }
        if ( TotalFiles > 10 )
            size = (10*y)/TotalFiles;
        else
            size = y - 1;
        v_Rectangle( x1, yi, x2, yi + size, OutLineColor );
        oyi = yi;
        m_ShowMouse;
    } else if ( msg->mouButton == LHOLD && size != (y-1) ) {
        my = objMouseY - myOffSet;
        if ( Scrolling == FALSE ) {
            Scrolling = TRUE;
            m_MouseWindow( x1, y1, x2, t.y2()-1 );
            if ( my >= yi && my <= yi + size )
                myOffSet = objMouseY - yi - 1;
            else
                myOffSet = size / 2;
            my = objMouseY - myOffSet;
            dlg->CSetHoldMsgCnt( FALSE );
        }
        if ( (my + size) > t.y2() )
            my = t.y2()-size;
        FileIndex = ((my - y1) * TotalFiles)/y;
        if ( FileIndex < 0 )
            FileIndex = 0;
        else if ( FileIndex + 10 >= TotalFiles )
            FileIndex = TotalFiles - 10;
        RedrawEntries();
    } else if ( msg->mouButton & CLICK ) {
        Scrolling = FALSE;
        m_MouseWindow( 0, 0, v_Width -1, v_Height - 4 );
        dlg->CSetHoldMsgCnt( TRUE );
        myOffSet = 0;
    }
    return 0;
}

//
// Object ID Name: SCROLL_DOWN_ID
//
// Object ID Name: SCROLL_UP_ID
//
DeclareObjH( ScrollBtn )
{
    Button &t = *((Button*)msg->ToObj);
    if ( !isClick( msg, 0 ) && msg->mouButton == LHOLD )
        if ( friction.peekTime() >= 1 )
            friction.start();
        else
            return 0;
    else if ( !(msg->mouButton & CLICK) )
        return 0;

    if ( t.CGetID() == SCROLL_UP_ID ) {
        if ( FileIndex > 0 ) FileIndex --;
        else return 0;
    } else {
        if ( FileIndex+10 < TotalFiles ) FileIndex ++;
        else return 0;
    }
    RedrawEntries();
    return 0;
}


//
// Object ID Name: DLG_TITLE_ID
//
DeclareObjH( Title )
{
    ScrObj &t = *(ScrObj*)msg->ToObj;
    if ( msg->actId == REDRAW ) {
        int x = v_CenterMsgX( t.x1(), t.x2(), ActionString ),
            y = v_CenterMsgY(t.y1(), t.y2());
        x = v_PrintMsg( x, y, TextColor, FaceColor, ActionString );
        m_ShowMouse;
    }
    return 0;
}


DeclareObjH( GoDlgGetFile );

#include "filedlg.src"

    static installed = 0;
    static Object *reply = 0;
    static Button *GFOKButton;

static void _pascal_ InstallDialog( void )
{
    if ( dlg == 0 ) {
        dlg = new Dialog(60, 56, 203, 232, 0, 126 );
        if ( !dlg ) {
            v_PrintMsg( 0, 0, TextColor, FaceColor, "New Failed!");
            return;
        }
        filedlg_SetupObjects(*dlg);
        GFOKButton = (Button*)dlg->CGetObject( FDLG_OK_BUTTON_ID );
        FNBox = (StrBox*)dlg->CGetObject( FILE_MASK_ID );
        if ( ExportFileName[0] )
            FNBox->CSetName( ExportFileName );
        else
            FNBox->CSetName( WildCard );
        FileIndex = 0;
        TotalFiles = GetFiles( WildCard, fns );
        ViewMgr::CSendMsg( dlg, INSTALL_DIALOG, GFOKButton );
    }
}

static void _pascal_ NewWildCard( void )
{
    FileIndex = 0;
    if ( FNBox->CStr()) {
        strcpy(WildCard, FNBox->CStr());
        if ( !FileExist( WildCard )) {
            TotalFiles = GetFiles( WildCard, fns );
            RedrawEntries();
        }
    }
}

DeclareObjH( GoDlgGetFile )
{
    int i;

    ScrObj &t = *(ScrObj*)msg->ToObj;

    switch( msg->actId ) {
        case GET_FILE_NAME:
            InstallDialog();
            return 0;
        case INSTALL_DIALOG:
            if ( !installed ) {
                installed = 1;
                dlg->CSendMsg( dlg->CGetObject(DLG_TITLE_ID), REDRAW );
            }
            return 0;
        case DESTROY_DIALOG:
            if ( dlg ) {
                for( i = filedlg_BASE_ID; i < filedlg_MAX_ID; i ++ )
                    delete (dlg->CGetObject(i));
                delete dlg;
                dlg = 0;
                installed = 0;
            }
            return 0;
        case DIALOG_CONFIRM:
            if ( reply ) {
                if ( FNBox->CStr() )
                    strcpy(ExportFileName, FNBox->CStr());
                ViewMgr::CSendMsg( reply, FILE_DLG_REPLY );
                reply = 0;
            }
            return 0;
        case DIALOG_CANCEL:
            if ( reply ) {
                ViewMgr::CSendMsg( reply, FILE_DLG_CANCEL );
                reply = 0;
            }
            return 0;
        case YESNO_YES:
            ViewMgr::CSendMsg( dlg, DIALOG_CONFIRM, GFOKButton );
            return 0;
        case 0:
        case MOUSE_OFF:
            break;
        case KEY_DOWN:
            if ( t.CGetID() == FILE_MASK_ID &&
               msg->keyDown == ENTER ) {
                if ( !ChangeDrive())  NewWildCard();
            }
            break;
        default:
            return 0;
    }
    if ( dlg && isClick( msg, LEFT_PRESSED )) {
        switch( t.CGetID()) {
            case FILE_MASK_ID:
                if ( !ChangeDrive())  NewWildCard();
                break;
            case FDLG_OK_BUTTON_ID:
                if ( !ChangeDrive())
                    yesno( "Are you sure?(y/n)", GFOKButton );
                break;
            case FDLG_CANCEL_BUTTON_ID:
                ViewMgr::CSendMsg( dlg, DIALOG_CANCEL,
                                dlg->CGetObject(FDLG_CANCEL_BUTTON_ID));
                break;
            default:
                break;
        }
    }
    return 0;
}

void GetFileName( char *wc, Object *Caller, int SendId, char *ActStr )
{
    strcpy( WildCard, wc );
    reply = Caller;

    if ( ActStr == 0 ) {
        ActionString[0] = 0;
    } else
        strncpy( ActionString, ActStr, 24 );

    Message *msg = new Message( 0, SendId, Caller, 0, 0 );
    GoDlgGetFile( msg );
    delete msg;
}




//#define TEST
#ifdef TEST
#include "errcode.h"
ViewMgr mmgr;
DeclareObjH( DoIT )
{
    if ( isClick( msg, LEFT_PRESSED )) {
        GetFileName( "????????.???", msg->ToObj, SEND_SAVE_ACTID );
    } else if ( msg->mouButton == RCLICK ) {
        GetFileName( "????????.???", msg->ToObj, SEND_LOAD_ACTID );
    } else if ( msg->actId == FILE_DLG_REPLY ) {
        v_PrintMsg( 0, 14, TextColor, FaceColor, ExportFileName );
        sprintf (buffer, "coreleft: %lu bytes", coreleft () );
        v_PrintMsg( 0, 24, TextColor, FaceColor, buffer );
    }
    return 0;
}

DeclareObjH( Dos )
{
    if ( isClick( msg, LEFT_PRESSED )) {
        int result;

        _RestoreVideo;
        result = spawnle(P_WAIT, "c:\\command.com", NULL, NULL);
        if (result == -1)
        {
            perror("Error from spawnle");
            exit(SPAWN_ERROR);
        }
        _InitVideo;
        mmgr.CShowAllObject();
    }
    return 0;
}

main()
{
    _InitVideo;
    m_ShowMouse;

    Button m( 2, 2, 60, 12, DoIT, 1958, "Do It");
    Button d( 2, 30, 60, 40, Dos, 1968, "Dos");

    // Enter Main loop
    Timer::CInstallTimer();
    mmgr.CRegObject( &m );
    mmgr.CRegObject( &d );
    mmgr.CShowAllObject();
    mmgr.CMgrLoop();
    Timer::CRemoveTimer();

    _RestoreVideo;
    return (0);

}
#endif
