#include <stdlib.h>
#include <dir.h>
#include <process.h>
#include <ctype.h>
#include "vidinter.h"  // Video interface
#include "timer.h"     // Interrupt driven timer
#include "fileutil.h"    // Read and write files
#include "depend.h"    // OS dependent stuff
#include "button.h"    // Implements Push buttons
#include "viewmgr.h"   // Implements View manager
#include "actids.h"	   // Action ids
#include "dialog.h"    // Simple Dialog Handler
#include "strbox.h"    // Implements the string box
#include "chkbox.h"    // Implements the Check Box
#include "keys.h"      // Key codes
#include "sprdesg.h"   // Header file for this module
#include "sprdesg.id"  // Object IDs for all the objects in this module
#include "paldlg.h"    // Header file for the Pallete Editor module
#include "filedlg.h"   // Header file for the File Dialog module
#include "buffer.h"    // Common buffer (1024 bytes)
#include "xbmtools.h"  // XLib bitmap tools
#include "scalebmp.h"  // Scaling linear bitmaps
#include "savesize.h"  // Pick part of a sprite for saving to file
#include "scale1.h"
#include "errcode.h"
#include "xfill.h"
#include "mousecur.h"   // Different mouse cursor shapes
#include "spcanvas.h"
#include "sprviewr.h"
#include "grpxprim.h"


#ifdef __MSDOS__
#include <dos.h>       // Needed for msdos
#include <alloc.h>     // for coreleft
#include <conio.h>     // getch and kbhit
#endif

CRL CRLDev;

void _pascal_ CRL::CCPlotPoints( void )
{
#define ClipPixel( px, py ) if ( px >= ClipLt && px <= ClipRt \
    && py >= ClipTp && py <= ClipBt ) {\
        myC->PutPixelPos( px, py ); \
    }
    int xl = xCenter - x, yl = yCenter - y;
    int xr = xCenter + x, yr = yCenter + y;
    ClipPixel( xr, yr );
    ClipPixel( xl, yr );
    ClipPixel( xr, yl );
    ClipPixel( xl, yl );
    xl = xCenter - y, yl = yCenter - x;
    xr = xCenter + y, yr = yCenter + x;
    ClipPixel( xr, yr );
    ClipPixel( xl, yr );
    ClipPixel( xr, yl );
    ClipPixel( xl, yl );
}

void _pascal_ CRL::CDrawCircle( int xC, int yC, int rad,
                                zSprView *v, zCanvas *c )
{
    xCenter = xC, yCenter = yC;
    x = 0, y = rad;
    ClipLt = v->OrigX, ClipTp = v->OrigY;
    ClipRt = v->x1() + v->Width, ClipBt = v->y1() + v->Height;

    myC = c;

    CCPlotPoints();
    p = 1 - rad;

    while ( x < y ) {
        x += 1;
        if ( p > 0 ) {
            y -= 1;
            p = p + 2 * ( x - y ) + 1;
        } else {
            p = p + 2 * x + 1;
        }
        CCPlotPoints();
    }
}


void _pascal_ iswap( int *x, int *y )
{
    int temp;
    temp = *x;
    *x = *y;
    *y = temp;
}

void _pascal_ CRL::CDrawRect( int x1, int y1, int x2, int y2, zCanvas *c)
{
    if ( x1 > x2 )
        iswap( &x1, &x2 );
    if ( y1 > y2 )
        iswap( &y1, &y2 );

    for( int i = x1; i <= x2; i ++ ) {
        c->PutPixelPos( i, y1 );
        c->PutPixelPos( i, y2 );
    }
    for( i = y1; i <= y2; i ++ ) {
        c->PutPixelPos( x1, i );
        c->PutPixelPos( x2, i );
    }
}


#define ABS(a)   ((a < 0) ? -a : a)
#define SGN(a)   ((a < 0) ? -1 : 1)

void _pascal_ CRL::CDrawLine( int x1, int y1, int x2, int y2, zCanvas *c)
{
    int d, ax, ay, sx, sy, dx, dy;

    x = x1;
    y = y1;
    if ( x1 != x2 || y1 != y2 ) {
        dx = x2-x1;
        ax = ABS(dx) << 1;
        sx = SGN(dx);
        dy = y2-y1;
        ay = ABS(dy) << 1;
        sy = SGN(dy);

        if( ax > ay ) {
            d = ay - (ax >> 1);
            do {
                c->PutPixelPos( x, y );
                if( d >= 0 ) {
                    y += sy;
                    d -= ax;
                }
                x += sx;
                d += ay;
            } while( x != x2 );
        } else {
            d = ax - (ay >> 1);
            do {
                c->PutPixelPos( x, y );
                if( d >= 0 ) {
                    x += sx;
                    d -= ay;
                }
                y += sy;
                d += ax;
            } while( y != y2 );
        }
    }
    c->PutPixelPos( x, y );
}
