#include <stdlib.h>
#include <dir.h>
#include <process.h>
#include <ctype.h>
#include "vidinter.h"  // Video interface
#include "timer.h"     // Interrupt driven timer
#include "fileutil.h"    // some File routines, mainly Read and write files
#include "depend.h"    // OS dependent stuff
#include "button.h"    // Implements Push buttons
#include "viewmgr.h"   // Implements View manager
#include "actids.h"    // Message Action ids
#include "dialog.h"    // Simple Dialog Handler
#include "strbox.h"    // Implements the string box
#include "chkbox.h"    // Implements the Check Box
#include "keys.h"      // Key codes
#include "sprdesg.h"   // Header file for this module
#include "sprdesg.id"  // Object IDs for all the objects in this module
#include "paldlg.h"    // Header file for the Pallete Editor module
#include "filedlg.h"   // Header file for the File Dialog module
#include "buffer.h"    // Common buffer (1024 bytes)
#include "xbmtools.h"  // XLib bitmap tools
// #include "scalebmp.h"  // Scaling linear bitmaps(by me, not used)
#include "savesize.h"  // Pick part of a sprite for saving to file
#include "scale1.h"     // John A slagel's scaling routine headers
#include "errcode.h"    // Exit error codes
#include "xfill.h"      // Floodfill routine from XLIB
#include "mousecur.h"   // Different mouse cursor shapes
#include "spcanvas.h"   // Header for Canvases module
#include "sprviewr.h"   // Header for Sprite viewers module
#include "grpxprim.h"   // Header for Drawing circle, line, rect module
#include "smalview.h"   // Header for sprite library module
#include "sprini.h"     // Handles the .ini files for sprdesg

#ifdef __MSDOS__
#include <dos.h>       // Needed for msdos
#include <alloc.h>     // for coreleft
#include <conio.h>     // getch and kbhit
#endif

int help = 0;

void _pascal_ BlueOnGrey( void );
extern DeclareObjH( MakeHelp );
extern DeclareObjH( ShowHelp );

void _pascal_ SprMMgr::KeyBoardHandler( void )
{
    if ( kbhit() ) {
        int key = getch();
        if ( !key ) key = getch() + 0xff;
        switch ( key ) {
            case 314:
                help = 1;
                m_HideMouse;
                x_define_mouse_cursor( ScanCur, MouseCursorColor );
                break;
            case 315:
                BlueOnGrey();
                break;
            case 388:
                help = 2;
                break;
            case 389:
                stop = 1;
                break;
            default:
                CSendMsg( WorkObj, KEY_DOWN, 0, 0, key );
                break;
        }
    }
}

void _pascal_
SprMMgr::ObjHandler (void )
{
	// Check object to object communication
    if ( otoSender ) {
        static Object *tmpObjPtr;
        Message *tmpMsgPtr = otoSender;

        AdvanceMsg();
        if ( !otoSender ) otoTail = 0;

        tmpObjPtr =(Object*) tmpMsgPtr->ToObj;
        if ( tmpObjPtr ) {
            if ( help == 1 )
                ShowHelp( tmpMsgPtr );
            else if ( help == 2 )
                MakeHelp( tmpMsgPtr );
            else
                tmpObjPtr->CallBack( tmpMsgPtr );
        }

        delete tmpMsgPtr;
    } else {
        m_MUSTShowMouse;    // Guarrantee work! no change will be made to this
        KeyBoardHandler();
        MouseHandler();
    }
}

//////////////////////////////////////////////////////////////////////////
//                                                                      //
// Global Variables                                                     //
//                                                                      //
//////////////////////////////////////////////////////////////////////////
Timer friction;         // Keep things at a reasonable speed
Timer friction2;
SprMMgr mmgr;                // Main Object Manager
BYTE SystemPal[7][3];      // System Pallete. Colors 1-7
int ViewerCount = 4;       // Number of viewers on screen
// Current pen color for left/right mouse button
WORD LeftColor, RightColor = 0;
WORD tempLeftColor, tempRightColor = 0;  // use for navigating colors
ScrObj *MouButColor;       // Object showing pen colors for l/r mouse button
ScrObj *ColorChooserObj;   // Object for picking colors out of the 256
char _FAR_ *CanvasData = 0;     // Memory use for working with sprites
char _FAR_ *DataConvert = 0;
char _FAR_ *Clipboard = 0;
char _FAR_ *MemPoolView = 0;    // Pool of memory split up for above.
char _FAR_ *MemPoolMisc = 0;
// Indicating which of the four sprite is being edit(use by the indicators)
// These ===ARE=== ****NOT**** ARRAY indexes.  They are object ids.
int LeftID = SPR_PREV_INDIC_ID, RightID = SPR_1_INDIC_ID;

// actually Indicating which of the four spirte is being edit.  Array indexes.
int LeftViewIndex = 0, RightViewIndex = 1;
//////////////////////////////////////////////////////////////////////////
//                                                                      //
// Done Global Variables                                                //
//                                                                      //
//////////////////////////////////////////////////////////////////////////


char * _pascal_ GetFileNameFromPath( char *path )
{
    if ( path ) {
        int i = strlen( path );
        while( i && path[--i] != '\\' );
        return (path+i+1);
    }
    return 0;
}



// A simple struct to hold lines of text
typedef struct zText
{
    char *str;
} Text;

static Text InfoText[] = {
#include "info.doc"
};


#define MAX_BUTTON_IMGS 22
static char _FAR_ *BtnImgs[MAX_BUTTON_IMGS];

// Turn on this to load button icons as individual files.
// #define _LOAD_ICONS_AS_FILES_

// These are the file names of each icon.  Use in the sprite library also.

static Text BtnImgNames[] = {
    { "load" }, { "save" }, { "undo" }, { "dosprmpt" }, { "pencil" },
    { "circle" }, { "line" }, { "rect" }, { "clear" },  { "colorsc" },
    { "rotate_l" }, { "rotate_r" }, { "fliph" }, { "flipv" },
    { "fill" }, { "sprsize" }, {"Animate"}, {"Swapclrs"},
    { "smrarrow" }, { "smlarrow" }, { "putarw" }, { "getarw" }
};

#define LoadImg         BtnImgs[0]
#define SaveImg         BtnImgs[1]
#define UndoImg         BtnImgs[2]
#define DosPrmptImg     BtnImgs[3]
#define PencilImg       BtnImgs[4]
#define CircleImg       BtnImgs[5]
#define LineImg         BtnImgs[6]
#define RectImg         BtnImgs[7]
#define ClearImg        BtnImgs[8]
#define ColorScanImg    BtnImgs[9]
#define RotateLtImg     BtnImgs[10]
#define RotateRtImg     BtnImgs[11]
#define FlipHImg        BtnImgs[12]
#define FlipVImg        BtnImgs[13]
#define FillImg         BtnImgs[14]
#define SprSizeImg      BtnImgs[15]
#define AnimateImg      BtnImgs[16]
#define SwapColorImg    BtnImgs[17]
#define SmallRtArwImg   BtnImgs[18]
#define SmallLtArwImg   BtnImgs[19]
#define PutArwImg       BtnImgs[20]
#define GetArwImg       BtnImgs[21]

//
// Local Function Protypes
//

static void _pascal_ SetLocalPtrs( ViewMgr &mgr )
{
    SprView[0] = (zSprView*)mgr.CGetObject( SPRITE_VIEW_PREV_ID );
    SprView[1] = (zSprView*)mgr.CGetObject( SPRITE_VIEW_1_ID );
    SprView[2] = (zSprView*)mgr.CGetObject( SPRITE_VIEW_2_ID );
    SprView[3] = (zSprView*)mgr.CGetObject( SPRITE_VIEW_ANIM_ID );
    SprView[0]->IndicObj = (ScrObj*)mgr.CGetObject( SPR_PREV_INDIC_ID );
    SprView[1]->IndicObj = (ScrObj*)mgr.CGetObject( SPR_1_INDIC_ID );
    SprView[2]->IndicObj = (ScrObj*)mgr.CGetObject( SPR_2_INDIC_ID );
    SprView[3]->IndicObj = (ScrObj*)mgr.CGetObject( SPR_ANIM_INDIC_ID );
    for( int i = 0; i < 4; i ++ )
        SprView[i]->Idx = i;
    Canvas[0] = (zCanvas*)mgr.CGetObject( CANVAS_1_ID );
    Canvas[1] = (zCanvas*)mgr.CGetObject( CANVAS_2_ID );
    Canvas[0]->Viewer = SprView[0];
    Canvas[1]->Viewer = SprView[1];
    Canvas[0]->AdjustPixSize();
    Canvas[1]->AdjustPixSize();
    SprView[0]->myCanvas = Canvas[0];
    SprView[1]->myCanvas = Canvas[1];
    MouButColor = (ScrObj*)mmgr.CGetObject( MOU_BUT_COLOR_ID );
    ColorChooserObj = (ScrObj*)mmgr.CGetObject( COLOR_CHOOSER_ID );
}

void _pascal_ MouseHVSpeed( int h, int v )
{
	_asm {
		mov ax, 15
        mov cx, h
        mov dx, v
		int 33h
	}
}

static char _FAR_ *AllocateBuffer( size_t size )
{
    char _FAR_ *t = new char _FAR_[size];
    if ( !t ) {
        printf("Error! Alocate %d bytes mem failed.\n", size);
        perror( "Bogus");
        exit(ALLOCATE_BUFFER_ERROR);
    }
    return t;
}

static void _pascal_ ResetButtonImgsColor( void )
{
    for( int i = 0; i < MAX_BUTTON_IMGS; i ++ ) {
        SwapColor( (BYTE _FAR_*)BtnImgs[i], DefRed, v_cRed );
        SwapColor( (BYTE _FAR_*)BtnImgs[i], DefBlue, v_cBlue );
        SwapColor( (BYTE _FAR_*)BtnImgs[i], DefGreen, v_cGreen );
        SwapColor( (BYTE _FAR_*)BtnImgs[i], DefBlack, v_cBlack );
        SwapColor( (BYTE _FAR_*)BtnImgs[i], DefWhite, v_cWhite );
    }
}

#ifdef _LOAD_ICONS_AS_FILES_
static void _pascal_ LoadButtonImgs( int Reload = 0 )
{
    strcpy( buffer, WorkDir );
    char *temp = buffer + strlen(buffer);
    for( int i = 0; i < MAX_BUTTON_IMGS; i ++ ) {
        sprintf( temp, "icons\\%s.pbm", BtnImgNames[i].str );
        int ImgSize;
        if ( Reload ) {
            ImgSize = v_BMPsize( PBMWidth( BtnImgs[i]),
                                 PBMHeight(BtnImgs[i]));
            BtnImgs[i] = readfile( buffer, ImgSize, BtnImgs[i]);
        } else
            BtnImgs[i] = readfile( buffer, ImgSize );
        if ( BtnImgs[i] == 0 ) {
            printf( "Error! Bogus. Can't load file %s\n", buffer );
            printf( "Bailing!!\n");
            exit(LOAD_BTN_IMAGE_ERROR);
        }
    }
}
#else
static void _pascal_ LoadButtonImgs( int = 0)
{
    static zSmallView Icons;

    // Merge file name with working directory.
    sprintf( buffer, "%sicons.slb", WorkDir );
    Icons.SetLibName( buffer );
    if ( !Icons.LoadLib() ) {
        printf("Error opening lib.");
        exit(235);
    }
    for( int i = 0; i < MAX_BUTTON_IMGS; i ++ ) {
        sprintf( buffer, "%s.pbm", BtnImgNames[i].str );
        BtnImgs[i] = Icons.GetSprite( buffer );
        if ( !BtnImgs[i]) {
            printf( "Error! Bogus. Can't load file %s\n", buffer );
            printf( "Bailing!!\n");
            exit(LOAD_BTN_IMAGE_ERROR);
        }
    }
}
#endif

static void _pascal_ Initialize( void )
{
    static int InitOne = 0;

    // Must reset default system colors before loading from ini file.
    v_ResetColors();
    LoadINIFile();

    if ( !InitOne ) {
        InitOne = 1;
        char *temp;
        MemPoolView = temp = AllocateBuffer( (64*64+2) * 6 );
        for( int i = 0; i < 4; i++ ) {
            SprView[i]->data = temp;
            PBMByteWidth(temp) = 0;
            temp += (64*64+2);
        }
        PBMByteWidth( temp ) = 0;
        Canvas[0]->UndoData = temp;

        temp += (64*64+2);
        PBMByteWidth( temp ) = 0;
        Canvas[1]->UndoData =  temp;

        MemPoolMisc = temp = AllocateBuffer( (128*128+2) * 3 );

        CanvasData = temp;
        temp += (128*128+2);
        DataConvert = temp;
        temp += (128*128+2);
        Clipboard = temp;
        LoadButtonImgs();
    } else
        LoadButtonImgs(1);

    _InitVideo;
    v_PutPal( RawPalData );

    ResetButtonImgsColor();

    v_RectFill(0, 0, v_Width, v_Height, FaceColor );
    mmgr.CShowAllObject();

    m_MouseWindow( 0, 0, v_Width-1, v_Height-4 );
    MouseHVSpeed();
}

DeclareObjH( ShowImage )
{
    if ( msg->actId != REDRAW ) return 0;
    ScrObj &t = *(ScrObj*)(msg->ToObj);
    char _FAR_ *ImgPtr = 0;
    switch( t.CGetID()) {
        case LOAD_SPR_ID:
            ImgPtr = LoadImg;
            break;
        case SAVE_SPR_ID:
            ImgPtr = SaveImg;
            break;
        case PENCIL_ID:
            ImgPtr = PencilImg;
            break;
        case DOSSHELL_ID:
            ImgPtr = DosPrmptImg;
            break;
        case UNDO_ID:
            ImgPtr = UndoImg;
            break;
        case CLEAR_ID:
            ImgPtr = ClearImg;
            break;
        case LINE_ID:
            ImgPtr = LineImg;
            break;
        case CIRCLE_ID:
            ImgPtr = CircleImg;
            break;
        case RECT_ID:
            ImgPtr = RectImg;
            break;
        case FILL_ID:
            ImgPtr = FillImg;
            break;
        case COLOR_SCAN_ID:
            ImgPtr = ColorScanImg;
            break;
        case FLIP_HOR_ID:
            ImgPtr = FlipHImg;
            break;
        case FLIP_VER_ID:
            ImgPtr = FlipVImg;
            break;
        case ROTATE_LEFT_ID:
            ImgPtr = RotateLtImg;
            break;
        case ROTATE_RIGHT_ID:
            ImgPtr = RotateRtImg;
            break;
        case RESIZE_ID:
            ImgPtr = SprSizeImg;
            break;
        case ANIMATION_ID:
            ImgPtr = AnimateImg;
            break;
        case SWAP_COLOR_ID:
            ImgPtr = SwapColorImg;
            break;
        case SMALV_LEFT_ID:
            ImgPtr = SmallLtArwImg;
            break;
        case SMALV_RIGHT_ID:
            ImgPtr = SmallRtArwImg;
            break;
        case GET_SMALL_SPR_ID:
            ImgPtr = GetArwImg;
            break;
        case PUT_SMALL_SPR_ID:
            ImgPtr = PutArwImg;
            break;
        default:
            return 0;
    }
    if ( ImgPtr ) {
        int x, y;
        x = ( t.x2() - t.x1() - PBMWidth(ImgPtr)+1)/2+t.x1();
        y = ( t.y2() - t.y1() - PBMHeight(ImgPtr)+1)/2+t.y1();
        v_PutMaskedbmp( x, y, ImgPtr );
    }
    return 0;
}

void _pascal_ SwapColor( BYTE _FAR_ *data, int color1, int color2 )
{
    if ( color1 == color2 ) return;
    size_t size = v_BMPsize( PBMWidth(data), PBMHeight(data) );
    data += 2;
    for( size_t i = 0; i < size; i ++ ) {
        if ( data[i] == (BYTE)color1 )
            data[i] = (BYTE)color2;
    }
}

//
// Object ID Name: COLOR_CHOOSER_ID
// Display all the colors that can be select
//
    static int xs, ys, xi, yi, x1, y1, x2, y2;
    static int Moving = 0, oldx = -1, oldy = -1;
    static int MarkX = -1, MarkY = -1, MarkI = -1;
    static WORD oldColor = 0;

static void _pascal_ Convert2XY( int *x, int *y, int c )
{
    *x = (c % 16) * xs + x1;
    *y = (c / 16) * ys + y1;
}

static int _pascal_ MarkColor( int x, int y )
{
    BYTE RGBVal[3];
    m_HideMouse;
    int mcolor = v_GetPixel( x, y );
    x_get_pal_raw( RGBVal, 1, mcolor );
    int a = max( (int)RGBVal[0], (int)RGBVal[1]);
    a = max( a, (int)RGBVal[2]);
    if ( a < 55 )
        v_Rectangle( x, y, x+xi, y+yi, v_cWhite );
    else
        v_Rectangle( x, y, x+xi, y+yi, v_cZero );
    return mcolor;
}


static void _pascal_ SelectNewColor( int x, int y )
{
    if ( MarkX != x || MarkY != y )
        oldColor = MarkColor( x, y );
    else
        oldColor = MarkI;
    oldx = x;
    oldy = y;
}

static void _pascal_ DrawColors( ScrObj &t )
{
    int i, j, w, h, ow, oh, color;
    color = 0;
    m_HideMouse;

    t.CClrContext();      // Clear area
    w = t.x2() - t.x1() - 1; // minus 1 because edges are exclusive
    h = t.y2() - t.y1() - 1;
    xs = w / 16; // Sixteen colors each row and column
    ys = h / 16; // This will give us the space to display each color
    if ( xs < 1 || ys < 1 )
        return;    // Not enough space
    // Find the exact rectangle to display our colors
    x1 = t.x1() + 1;
    y1 = t.y1() + 1;
    x2 = xs * 16;
    y2 = ys * 16;
    ow = (w-x2)/2;  // Do we have any extra space to center the rect?
    oh = (h-y2)/2;
    x2 += t.x1();   // Add offset x and y coordinate
    y2 += t.y1();
    if ( ow > 0 ) { // Check if need to center the rect
        x2 += ow;   // in the object.
        x1 += ow;
    }
    if ( oh > 0 ) {
        y2 += oh;
        y1 += oh;
    }
    xi = xs - 1;    // Increment is the width/heigh of rect - 1
    yi = ys - 1;    // since left edge is inclusive
    // Draw the colors on the color chooser area, center if necessaried
    for( j = y1; j <= y2; j += ys ) {
        for( i = x1; i <= x2; i += xs ) {
            if ( color > 255 )
                return;
            v_RectFill( i, j, i+xs, j+ys, color );
            if ( color == LeftColor )
                SelectNewColor( i, j );
            color ++;
        }
    }
    m_ShowMouse;
}

static void _pascal_ StopPickingColor( void )
{
    ViewMgr::CSendMsg( MouButColor, MOU_BUT_COLOR_NONAVIGATE );
    ViewMgr::CSendMsg( MouButColor, REDRAW );
    Moving = 0;
    m_MouseWindow( 0, 0, v_Width - 1, v_Height - 4 );
}

DeclareObjH( ColorChooser )
{
    ScrObj &t = *((ScrObj*)msg->ToObj);

    switch( msg->actId ) {
        case CHANGE_RIGHT_MARK_COLOR:
        case CHANGE_LEFT_MARK_COLOR:
            int x, y;
            m_HideMouse;
            v_Rectangle( oldx, oldy, oldx+xi, oldy+yi, oldColor );
            if ( msg->actId == CHANGE_RIGHT_MARK_COLOR )
                Convert2XY( &x, &y, RightColor );
            else
                Convert2XY( &x, &y, LeftColor );
            SelectNewColor( x, y );
            m_ShowMouse;
            return 0;
        case REDRAW:
            MarkX = MarkY = MarkI = -1;
            DrawColors(t);
            return 0;
        case MOUSE_OFF:
            m_ShowMouse;
            return 0;
        case CANCEL_CLICK:
            if ( Moving ) {
                tempLeftColor = LeftColor;
                tempRightColor = RightColor;
                // This function will send a turn off navigation
                // mode and a redraw message to MouButColor
                StopPickingColor();
            }
            return 0;
        case MARK_COLOR:
            if ( MarkX < 0 ) {
                MarkX = oldx;
                MarkY = oldy;
                MarkI = oldColor;
            }
            return 0;
        case UNMARK_COLOR:
            if ( MarkX > -1 && MarkI != oldColor ) {
                m_HideMouse;
                v_Rectangle( MarkX, MarkY, MarkX + xi, MarkY + yi, MarkI );
                m_ShowMouse;
            }
            MarkX = MarkY = MarkI = -1;
            return 0;
        default:
            break;
    }
    int x = objMouseX - x1, y = objMouseY - y1;
    x = (x/xs)*xs + x1;
    y = (y/ys)*ys + y1;
    switch ( msg->mouButton ) {
    case LHOLD:
    case RHOLD:
        if ( oldx != x || oldy != y ) {
            if ( !Moving ) {
                m_HideMouse;
                m_MouseWindow( t.x1()+1, t.y1()+1, t.x2()-1, t.y2()-1 );
                m_UpdateMouse;
                x = objMouseX - x1, y = objMouseY - y1;
                x = (x/xs)*xs + x1;
                y = (y/ys)*ys + y1;
                if ( oldx > -1 && MarkI != oldColor ) {
                    v_Rectangle( oldx, oldy, oldx+xi, oldy+yi, oldColor );
                    Moving = 1;
                }
                SelectNewColor(x, y);
                // Set mouse button color displayer to navigate mode.
                ViewMgr::CSendMsg( MouButColor, MOU_BUT_COLOR_NAVIGATE );
                m_ShowMouse;
            } else if ( oldx != x || oldy != y ) {
                if ( MarkI != oldColor ) {
                    m_HideMouse;
                    v_Rectangle( oldx, oldy, oldx+xi, oldy+yi, oldColor );
                }
                SelectNewColor(x, y);
            }
            if ( msg->mouButton == LHOLD )
                tempLeftColor = oldColor;
            else
                tempRightColor = oldColor;
            ViewMgr::CSendMsg( MouButColor, REDRAW );
        }
        break;
    case LCLICK:
    case RCLICK:
        // Note we do not change the temp colors here, therefore changing
        // the drawing colors
        ViewMgr::CSendMsg( MouButColor, MOU_BUT_CHANGE_COLOR );
        StopPickingColor();
        break;
    default:
        break;
    }
    return 0;
}

//
// Object ID Name: MOU_BUT_COLOR_ID
// Display correspond color
// for each mouse button
//
static void _pascal_ ShowColor( int x, int y, int color )
{
    sprintf( buffer, "%3u", color );
    v_ShowMsg( x, y, TextColor, FaceColor, buffer );
    y += v_CharHeight() + 1;
    v_RectFill( x, y, x+17, y + 6, color );
    v_Rectangle( x, y, x+17, y + 6, v_cWhite );
}

static void  _pascal_ swap( WORD *a, WORD *b )
{
    WORD t = *a;
    *a = *b;
    *b = t;
}

DeclareObjH( MouseButColor )
{
    ScrObj &t = *((ScrObj*)msg->ToObj);
    static BBOOL Navigating = FALSE;
    WORD *l, *r;
    if ( Navigating ) {
        l = &tempLeftColor;
        r = &tempRightColor;
    } else {
        l = &LeftColor;
        r = &RightColor;
    }
    if ( isClick( msg, 0 )) {
        if ( msg->mouButton == LCLICK ) {  // Swap left and right colors
            swap( &tempLeftColor, &tempRightColor );
            swap( &LeftColor, &RightColor );
            ViewMgr::CSendMsg( &t, REDRAW );  // Self Redraw
            return 0;
        } else {
            msg->actId = -1;  // -1 is used as a flexible and universal
                              // Action ID to call objects.  It usually
                              // serve as the entry action for an object
                              // which is created when its handler is
                              // called.  This kind of object cannot receive
                              // message from others until it receives -1.
            EditPalette(msg);
        }
    } else switch( msg->actId ) {
        case MOU_BUT_COLOR_NAVIGATE:
            Navigating = TRUE;
            break;
        case MOU_BUT_COLOR_NONAVIGATE:
            Navigating = FALSE;
            break;
        case MOU_BUT_CHANGE_COLOR:
            LeftColor = tempLeftColor;
            RightColor = tempRightColor;
            break;
        case REDRAW:
            m_HideMouse;
            ShowColor( t.x1()+2, t.y1()+1, *l );
            ShowColor( t.x1()+18+7, t.y1()+1, *r );
            m_ShowMouse;
            break;
        default:
            break;
    }
    return 0;
}

//
// Object ID Name: SPR_PREV_INDIC_ID
//
// Object ID Name: SPR_1_INDIC_ID
//
// Object ID Name: SPR_2_INDIC_ID
//
// Object ID Name: SPR_ANIM_INDIC_ID
//
DeclareObjH( Indicator )
{
    ScrObj &t = *((ScrObj*)msg->ToObj);

    if ( msg->actId == REDRAW ) {
        if (t.CMouseOnMe())
            m_HideMouse;
        if ( LeftID == t.CGetID())
            t.CClrContext( v_cWhite );
        else if ( RightID == t.CGetID())
            t.CClrContext( v_cBlue );
        else
            t.CClrContext();
        m_ShowMouse;
        return 0;
    }
    for ( int Idx = 0; Idx < ViewerCount; Idx ++ )
        if ( SprView[Idx]->IndicObj->CGetID() == t.CGetID())
            SprView[Idx]->SelectViewer( msg->mouButton );
    return 0;
}

//
// Object ID Name: LOAD_SPR_ID
//
// Object ID Name: SAVE_SPR_ID
//

static void _pascal_ LoadSprite( char *fn, int Idx )
{
    if ( Idx < 0 || Idx >= ViewerCount ) return;
    int MaxSprSize = 64*64+2;
    char _FAR_ *r = readfile( fn, MaxSprSize, SprView[Idx]->data );
    if ( r ) {
        strncpy( SprView[Idx]->Name, fn, MAXFILE+MAXEXT );
        SprView[Idx]->myCanvas->SaveUndo();
        SprView[Idx]->SetNewBuf();
    } else {
        v_PrintMsg( 0, 225, TextColor, FaceColor,
            "Load sprite failed.  Press a key.");
        getch();
    }
}

void _pascal_ SaveSprite( char *fn, int Idx, char _FAR_ *data, int w, int h )
{
    if ( Idx < 0 || Idx >= ViewerCount ) return;
    WORD size = v_BMPsize( w, h );
    if ( writefile( fn, data, size) < 1 ) {
        v_PrintMsg( 0, 225, TextColor, FaceColor,
            "Save sprite failed.  Press a key.");
        getch();
    }
}

DeclareObjH( LoadSave )
{
    Button &t = *((Button*)msg->ToObj );
    char buf[20];

    static int x1 = 1, y1 = 1, w = 32, h = 32;
    static int saving = 0, which = -1;

    Button::CAction( msg, 0 );  // show clicking on the button
    switch ( msg->actId ) {
        case REDRAW:
            ShowImage(msg);
            m_ShowMouse;
            break;
        case FILE_DLG_REPLY:
            if ( t.CGetID() == LOAD_SPR_ID ) {
                LoadSprite( ExportFileName, which );
                which = -1;
            } else {
                saving = 1;
                if ( PBMByteWidth(SprView[which]->data)) {
                    if ( w > SprView[which]->Width )
                        w = SprView[which]->Width;
                    if ( h > SprView[which]->Height )
                        h = SprView[which]->Height;
                    Canvas[0]->CSetLower( Canvas[0]->x1()+128,
                                          Canvas[0]->y1()+128);
                    GetSaveSize( &t, GET_SAVE_SIZE,
                                Canvas[0],
                                SprView[0],
                                SprView[which]->data,
                                CanvasData,
                                &x1, &y1, &w, &h );
                }
            }
            break;
        case CONFIRM_GET_SIZE:
            if ( saving )
                SaveSprite( ExportFileName, which, CanvasData, w, h );
            // No break, continues
        case CANCEL_GET_SIZE:
            if ( saving ) {
                Canvas[0]->Adjusted = TRUE;
                SprView[0]->CShowObject();
                ViewMgr::CSendMsg( Canvas[0], REDRAW );
            }
            saving = 0;
            which = -1;
            break;
        default:
            break;
    }
    if ( msg->mouButton & CLICK ) {

        if ( msg->mouButton == RCLICK ) {
            strcpy( buf+5, "right Sprite");
            which = RightViewIndex;
        } else {
            strcpy( buf+5, "left Sprite");
            which = LeftViewIndex;
        }

        strncpy( ExportFileName, SprView[which]->Name,
                MAXFILE+MAXEXT);
        if ( t.CGetID() == LOAD_SPR_ID ) {
            sprintf(buf, "Load %s", buf+5);
        } else {
            sprintf(buf, "Save %s", buf+5);
        }
        GetFileName( "*.pbm", &t, GET_FILE_NAME, buf );
    }
    return 0;
}

//
// Object ID Name: UNDO_ID
//
DeclareObjH( Undo )
{
    Button::CAction( msg, 0 );
    int ci;
    ShowImage(msg);
    if ( msg->mouButton == LCLICK ) {
        ci = 0;
    } else if ( msg->mouButton == RCLICK ) {
        ci = 1;
    } else
        return 0;

    Canvas[ci]->DoUndo();
    Canvas[ci]->Viewer->RedrawVC();
    return 0;
}

//
// Object ID Name: DOSSHELL_ID
//
DeclareObjH( DosShell )
{
    ShowImage(msg);
    if ( isClick( msg, LEFT_PRESSED)) {
        int result;

        _RestoreVideo;
        Chdir( OrigDir );
        result = spawnle(P_WAIT, "c:\\command.com", NULL, NULL);
        if (result == -1)
        {
            perror("Error from spawnle");
            exit(SPAWN_ERROR);
        }
        Chdir( WorkDir );
        Initialize();
    }
    return 0;
}


void _pascal_ BlueOnGrey( void )
{
    static int oldTextColor, oldFaceColor;
    static int Changed = 0;
    static BYTE TextColorPal[3], FaceColorPal[3];

    if ( Changed ) {
        x_put_pal_raw( TextColorPal, 1, oldTextColor );
        x_put_pal_raw( FaceColorPal, 1, oldFaceColor );
    } else {
        oldTextColor = TextColor;
        oldFaceColor = FaceColor;
        x_get_pal_raw( TextColorPal, 1, TextColor );
        x_get_pal_raw( FaceColorPal, 1, FaceColor );
        buffer[0] = buffer[1] = 0;
        buffer[2] = 63;
        x_put_pal_raw( (BYTE _FAR_*)buffer, 1, TextColor );
        buffer[0] = buffer[1] = buffer[2] = 32;
        x_put_pal_raw( (BYTE _FAR_*)buffer, 1, FaceColor );
    }
    Changed = !Changed;
}

BBOOL MakeHelpEnable = FALSE;

DeclareObjH( MakeHelp )
{
    int result;
    ScrObj &t = *(ScrObj*)msg->ToObj;

    if ( msg->mouButton & CLICK && MakeHelpEnable == TRUE ) {
        help = 0;
        _RestoreVideo;

        // Merge working path with filename.
        sprintf( buffer, "%ssprhelpt.tmp", WorkDir );

        FILE *f = fopen( buffer, "w" );
        if ( !f ) {
           perror( "Error opening file");
           getch();
        }
        fprintf( f, ".ID%d\n", t.CGetID());
        fprintf( f, ".CMT\n");
        fprintf( f, ".BMP\n");
        fprintf( f, ".BEGIN\n");
        fclose( f );
        result = spawnle(P_WAIT, "c:\\boot\\q.exe", "q.exe", buffer,
                                 NULL, NULL);
        if (result == -1)
        {
            perror("Error from spawnle");
            getch();
        }
        f = fopen( buffer, "r");
        if ( !f ) {
            perror( "Error opening file");
            getch();
        }
        sprintf( buffer, "%shelp.txt", WorkDir );
        FILE *o = fopen( buffer, "a+");
        if ( !o ) {
            perror( "Error opening help.txt");
            getch();
        }
        char *r;
        while( 1 ) {
            r = fgets( buffer, 1024, f );
            if ( !r ) break;
            fprintf( o, "%s", buffer );
        }
        fprintf( o, "~\n");
        fclose(f);
        fclose(o);

        Initialize();
        if ( CurrentMgr != &mmgr )
            ((ViewMgr*)CurrentMgr)->CShowAllObject();
    }
    if ( MakeHelpEnable == FALSE  ) help = 0;
// andout   x_define_def_mouse_cursor( MouseCursorColor );
    x_define_mouse_cursor( defCursor, MouseCursorColor );
    return 0;
}

#define HLP_X1 25
#define HLP_Y1 20
#define HLP_X2 25
#define HLP_Y2 20

void _pascal_ PrintHelp( FILE *f )
{
    char *r;
    buffer[0] = 0;
    int i = HLP_Y1+v_CharHeight()+4, len;
    while( 1 ) {
        r = fgets( buffer, 512, f );
        if ( buffer[0] == '~' ) break;
        if ( !r ) break;
        len = strlen( buffer ) - 1;
        buffer[len] = 0;
        v_PrintMsg( HLP_X1+5, i, TextColor, FaceColor, buffer );
        i += v_CharHeight();
    }
}

void _pascal_ PrintNoSection( void )
{
    v_PrintMsg( HLP_X1+10, HLP_Y1+v_CharHeight()+4, TextColor, FaceColor,
                "No help available for this." );
}

void _pascal_ PrintNoBegin( void )
{
    v_PrintMsg( HLP_X1+10, HLP_Y1+v_CharHeight()+4, TextColor, FaceColor,
                "Begin keyword missing. The help" );
    v_PrintMsg( HLP_X1+10, HLP_Y1+v_CharHeight()*2+1, TextColor, FaceColor,
                "file may be corrupt." );
}

DeclareObjH( ShowHelp );

static void _pascal_ PrintInfo( void )
{
    int i = HLP_Y1 + v_CharHeight() + 4, j = 0;
    while( InfoText[j].str ) {
        if ( InfoText[j].str[0] == '.' )
            i += 3;
        else {
            v_PrintMsg( HLP_X1+5, i, v_cBlue, FaceColor, InfoText[j].str );
            i += v_CharHeight();
        }
        j ++;
    }
}

static void _pascal_ DisplayText( char *titlestr, WORD id, int info = 0 )
{
    ViewMgr hlpMgr;
    Button HelpDone( HLP_X1+5, v_Height-HLP_Y2-v_CharHeight()-6,
                    v_Width-HLP_X2-5, v_Height-HLP_Y2-4, ShowHelp,
                    0xEA60, "I have finished reading this." );
    hlpMgr.CRegObject( &HelpDone );

    x_cp_vid_rect( 0, 0, v_Width, v_Height, 0, 0,
                   VisiblePageOffs, HiddenPageOffs, v_Width, v_Width );
    v_RectFill( HLP_X1, HLP_Y1,
                v_Width-HLP_X2, v_Height-HLP_Y2, FaceColor );
    v_Rectangle( HLP_X1, HLP_Y1,
                v_Width-HLP_X2-1, v_Height-HLP_Y2-1, BtnUpColor );
    v_Rectangle( HLP_X1+1, HLP_Y1+1,
                v_Width-HLP_X2, v_Height-HLP_Y2, BtnLowColor );
    v_PrintMsg( v_CenterMsgX( HLP_X1, v_Width-HLP_X2, titlestr),
                HLP_Y1+2, TextColor, FaceColor, titlestr);

    if ( !info ) {
        // Merge path with file.
        sprintf( buffer, "%shelp.txt", WorkDir );
        FILE *f = fopen( buffer, "r");
        char *r;
        if ( f ) {
            sprintf( buffer, "ID%u", id );
            r = FindString( '.', buffer+20, f, buffer );
            if ( r ) {
                r = FindString( '.', buffer, f, "BEGIN" );
                if ( r )
                    PrintHelp( f );
                else
                    PrintNoBegin();
            } else
                PrintNoSection();
            fclose( f );
        }
    } else {
        PrintInfo();
    }
    hlpMgr.CShowAllObject();
    hlpMgr.CMgrLoop();
    x_page_flip(0, 0);
    VideoPage = VisiblePageOffs;
}


DeclareObjH( ShowHelp )
{
    ScrObj &t = *(ScrObj*)msg->ToObj;

    if ( (WORD)t.CGetID() == 0xEA60 ) {
        Button::CAction( msg, LEFT_PRESSED );
    }
    if ( msg->mouButton & CLICK ) {
        m_HideMouse;
        help = 0;
//andout	x_define_def_mouse_cursor( MouseCursorColor );
	x_define_mouse_cursor(defCursor, MouseCursorColor );
	if ( (WORD)t.CGetID() == 0xEA60 ) { // 60000
            CurrentMgr->CStopLoop();
        } else {
            t.CRestoreOutLine();
            DisplayText( "-- Help --", t.CGetID() );
        }
    }
    return 0;
}

//
// Object ID Name: PENCIL_ID
//
// Object ID Name: LINE_ID
//
// Object ID Name: CIRCLE_ID
//
// Object ID Name: RECT_ID
//
// Object ID Name: FILL_ID
//
// Object ID Name: SWAP_COLOR_ID
//
DeclareObjH( SetEditType )
{
    Button &t = *(Button*)(msg->ToObj);
    Button::CAction( msg, 0 );
    ShowImage(msg);
    if ( msg->mouButton == LCLICK )
        Canvas[0]->penType = t.CGetID();
    else if ( msg->mouButton == RCLICK )
         Canvas[1]->penType = t.CGetID();
    return 0;
}

//
// Object ID Name: CLEAR_ID
//
DeclareObjH( Clear )
{
    zSprView *t;
    Button::CAction( msg, 0 );
    ShowImage(msg);
    if ( msg->mouButton == LCLICK ) {
        t = SprView[LeftViewIndex];
    } else if ( msg->mouButton == RCLICK ) {
        t = SprView[RightViewIndex];
    } else
        return 0;
    if ( PBMByteWidth(t->data )) {
        t->myCanvas->SaveUndo();
        PBMByteWidth(t->data) = 0;
        t->RedrawVC();
    }

    return 0;
}

//
// Object ID Name: COLOR_SCAN_ID
//
DeclareObjH( ColorScan )
{
    static BBOOL ScannerOn = FALSE;
    static int Color;
    ShowImage(msg);
    if ( isClick( msg, 0)) {
        if ( !ScannerOn ) {
            ScannerOn = TRUE;
            m_HideMouse;
            x_define_mouse_cursor( ScanCur, MouseCursorColor );
        } else
            ScannerOn = FALSE;
        mmgr.CSetHoldMsgCnt( !ScannerOn );
        mmgr.CSetObjectFocus( ScannerOn );
    } else if ( !ScannerOn )
        return 0;
    m_HideMouse;
    Color = v_GetPixel( m_MouseX, m_MouseY );
    sprintf( buffer, "Pixel Color: %3d", Color );
    if ( !ScannerOn ) {
        int actId;
        if ( msg->mouButton == LCLICK ) {
            tempLeftColor = Color;
            actId = CHANGE_LEFT_MARK_COLOR;
        } else {
            tempRightColor = Color;
            actId = CHANGE_RIGHT_MARK_COLOR;
        }
        m_HideMouse;
//andout	x_define_def_mouse_cursor( MouseCursorColor );
	x_define_mouse_cursor( defCursor, MouseCursorColor );
	mmgr.CSendMsg( MouButColor, MOU_BUT_CHANGE_COLOR );
        mmgr.CSendMsg( MouButColor, REDRAW );
        mmgr.CSendMsg( ColorChooserObj, actId );
        buffer[0] = 0;
    }
    v_ShowMsgLen( 0, 225, MSG_LEN, TextColor, FaceColor, buffer );
    m_ShowMouse;
    return 0;
}


//
// Object ID Name: ROTATE_LEFT_ID
//
// Object ID Name: ROTATE_RIGHT_ID
//
// Object ID Name: FLIP_HOR_ID
//
// Object ID Name: FLIP_VER_ID
//
DeclareObjH( FlipRotate )
{
    Button::CAction( msg, 0 ); // Show mouse action
    Button &t = *(Button*)(msg->ToObj);
    zSprView *tp;
    ShowImage(msg);
    if ( msg->mouButton == LCLICK ) {
        tp = SprView[LeftViewIndex];
    } else if ( msg->mouButton == RCLICK ) {
        tp = SprView[RightViewIndex];
    } else
        return 0;
    switch( t.CGetID()) {
        case FLIP_HOR_ID:
            tp->FlipHorizontal();
            break;
        case FLIP_VER_ID:
            tp->FlipVertical();
            break;
        case ROTATE_LEFT_ID:
            tp->RotateLeft();
            break;
        case ROTATE_RIGHT_ID:
            tp->RotateRight();
            break;
        default:
            break;
    }
    return 0;
}

//
// Object ID Name: RESIZE_ID
//
DeclareObjH( Resize )
{
    static int *nwp, *nhp, nw, nh;
    static int ci, on = 0;
    ShowImage(msg);
    if ( on ) {
        switch( msg->actId ) {
        case CONFIRM_SPR_WH:
            if ( nw != *nwp || nh != *nhp ) {
                Canvas[ci]->SaveUndo();
                Canvas[ci]->Viewer->SetWH( nw, nh );
                mmgr.CSendMsg( Canvas[ci], REDRAW );
            }
            // no break, continue to next case
        case CANCEL_SPR_WH:
            on = 0;
            break;
        default:
            break;
        }
        return 0;
    }
    Button::CAction( msg, 0 );
    if ( msg->mouButton & CLICK ) {
        if ( msg->mouButton == LCLICK ) {
            ci = 0;
        } else if ( msg->mouButton == RCLICK ) {
            ci = 1;
        } else
            return 0;
        on = 1;
        nwp = &(Canvas[ci]->Viewer->Width);
        nhp = &(Canvas[ci]->Viewer->Height);
        nw = *nwp, nh = *nhp;
        GetWH( (Object*)msg->ToObj, GET_SPR_WH, &nw, &nh, 64, 64,
               "Specify new sprite size." );
    }
    return 0;
}


//
// Object ID Name: QUIT_BUT_ID
//
DeclareObjH( QuitDesg )
{
    if ( msg->actId == YESNO_YES ) {
        mmgr.CStopLoop();
    } else if ( isClick( msg, LEFT_PRESSED)) {
        yesno( "Are you sure?", (Object*)msg->ToObj );
    }
    return 0;
}

//
// Object ID Name: ANIMATION_ID
//
DeclareObjH( Animate )
{
    ShowImage(msg);
    if ( isClick( msg, LEFT_PRESSED)) {
        InitAnimate();
    }
    return 0;
}
//
// Object ID Name: HELP_ID
//
DeclareObjH( Help )
{
    if ( isClick( msg, LEFT_PRESSED)) {
        help = 1;
        x_define_mouse_cursor( ScanCur, MouseCursorColor );
    }
    return 0;
}

//
// Object ID Name: INFO_ID
//
DeclareObjH( Info )
{
    Button &t = *(Button*)msg->ToObj;
    if ( isClick( msg, LEFT_PRESSED)) {
        t.CRestoreOutLine();
        m_HideMouse;
        DisplayText( "-- Info --", 0, 1 );
    }
    return 0;
}

#define _I_HAVE_MY_OWN_HANDLERS_
#include "sprdesg.src"

main( int argc, char **argv )
{
    if ( argc == 2 && strncmp( argv[1], "makehelp", 8 ) == 0 ) {
        MakeHelpEnable = TRUE;
    }

    sprdesg_SetupObjects(mmgr);
    SetLocalPtrs( mmgr );

    // Save Original path and get the working dir from the
    // command.

    InitDirs( argv[0] );

    Initialize();

    // Speed up the timer by a factor of 10
    if (!Timer::CInstallTimer()) {
        printf("Failed to install timer!\n");
        exit(TIMER_ERROR);
    }

    LeftColor = tempLeftColor = v_cWhite;

    // Enter Main loop

    mmgr.CMgrLoop();

    Timer::CRemoveTimer();

    _RestoreVideo;

    // Restore original drive and path
    PopDir(0);

    return( NO_ERROR );

}
