#ifndef _SPRLIBE_H_
#define _SPRLIBE_H_

#define SPRLIB_SIG 0xFE0A
#define SPRHEADER_SIG  0xAECF

class zSprHeader
{
private:
    WORD Size;
    char Name[MAXFILE+MAXEXT];
public:
    friend class zSprLib;
    _pascal_ zSprHeader( char *name = 0, WORD size=0 );
    _pascal_ ~zSprHeader() {}
};
class zSprData
{
private:
    zSprHeader SprHeader;
    char _FAR_ *Buf;
    zSprData *next;
public:
    friend class zSprLib;
    _pascal_ zSprData( void ):Buf(0),next(0) {}
    _pascal_ ~zSprData() { delete Buf; }
};


#define APPEND      1
#define REPLACE     2
#define INSERT      3

class zSprLib
{
private:
    int CurIdx;   // Use by SprExist only.
    zSprData *head, *cur, *tail;
    char LibName[MAXPATH];
    WORD _sig;
    int  SpriteCount;
    long FileLoc;
    int  opened;

    // Find the sprite in the list with sname, return 0 if not found.
    zSprData * _pascal_ FindSpr( char *sname, int &i );

public:
    _pascal_ zSprLib( char *name = 0 );
    _pascal_ ~zSprLib();


    void _pascal_ SetName( char *name ) { strncpy( LibName, name, MAXPATH);}
    // Open a new library.
    int _pascal_ Init( void );

    // Add a sprite to end of list and return that sprite's number.
    // Returns -1 if failed.
    int _pascal_ AddSpr( char _FAR_ *buf, WORD size, char *sname );
    int _pascal_ InsertSpr( char _FAR_ *buf, WORD size, char *sname,
                                     char *insName );
    int _pascal_ SwapSpr( char *sname1, char *sname2 );

    // Return index of the sprite if exist
    // otherwise returns -1
    // Also sets cur pointer to the sprite found, so if this function
    // returns non-negative number, an immediate call to PutCurSpr
    // will save the sprite just check.
    int _pascal_ SprExist( char *sname );

    // Idx must equal to CurIdx, which is set by SprExist above.
    // Will return -1 if failed.  Otherwise returns Idx back.
    // These two act like replacing.
    int _pascal_ PutCurSpr( int Idx, char _FAR_ *buf, WORD size );
    int _pascal_ PutSpr( char _FAR_ *buf, WORD size, char *sname );

    const char _FAR_ * _pascal_ GetSpr( char *sname, WORD &size );

    // Remove Sprite with name sname from memory.
    // returns index of sprite remove or -1 on failure
    int _pascal_ DelSpr( char *sname );

    int _pascal_ HowMany( void ) { return SpriteCount; }

    // Write the whole lib to file.
    // returns -1 if failed or total number of sprites saved to lib
    int _pascal_ WriteLib( void );
    // Read a Sprite Lib file.
    // If Type is:
    //     APPEND -> Ignore BeforeWhich and add all sprites in the lib to
    //               tail.
    //     REPLACE ->Replace all sprites after and including sprite
    //               BeforeWhich - 1
    //     INSERT -> Insert before BeforeWhich
    //
    int _pascal_ ReadLib( int Type = APPEND, int BeforeWhich = 0 );

    // Read a single sprite into the lib.  Type & Bwhich similar to above.
    int _pascal_ ReadSpr( char *sname, int Type = APPEND, int BWhich = 0 );

    // Write a single sprite from lib to a file as fname.
    // return the number of the sprite written.
    int _pascal_ WriteSpr( char *sname, char *fname );

    // Get name of first sprite
    char * _pascal_ GetFirst( void );

    // Get name of next sprite(or the current sprite.)
    char * _pascal_ GetNext( void );

    char * _pascal_ GetNth( int i );
};

#endif
