#include <stdlib.h>
#include <dir.h>
#include <process.h>
#include <ctype.h>
#include "vidinter.h"  // Video interface
#include "timer.h"     // Interrupt driven timer
#include "fileutil.h"    // Read and write files
#include "depend.h"    // OS dependent stuff
#include "button.h"    // Implements Push buttons
#include "viewmgr.h"   // Implements View manager
#include "actids.h"	   // Action ids
#include "dialog.h"    // Simple Dialog Handler
#include "strbox.h"    // Implements the string box
#include "chkbox.h"    // Implements the Check Box
#include "keys.h"      // Key codes
#include "sprdesg.h"   // Header file for this module
#include "sprdesg.id"  // Object IDs for all the objects in this module
#include "paldlg.h"    // Header file for the Pallete Editor module
#include "filedlg.h"   // Header file for the File Dialog module
#include "buffer.h"    // Common buffer (1024 bytes)
#include "xbmtools.h"  // XLib bitmap tools
#include "scalebmp.h"  // Scaling linear bitmaps
#include "savesize.h"  // Pick part of a sprite for saving to file
#include "scale1.h"
#include "errcode.h"
#include "xfill.h"
#include "mousecur.h"   // Different mouse cursor shapes
#include "spcanvas.h"
#include "sprviewr.h"
#include "grpxprim.h"
#include "sprdesg.h"


#ifdef __MSDOS__
#include <dos.h>       // Needed for msdos
#include <alloc.h>     // for coreleft
#include <conio.h>     // getch and kbhit
#endif

zSprView *SprView[4];

//
// Object ID Name: SPRITE_VIEW_PREV_ID
// View sprite before
// sprite in SpriteView1
//
// Object ID Name: SPRITE_VIEW_1_ID
// View sprite in Canvas1
//
// Object ID Name: SPRITE_VIEW_2_ID
// View sprite in Canvas2
//
// Object ID Name: SPRITE_VIEW_ANIM_ID
// View a moving version of the current sprites
//

int _pascal_ zSprView::CheckMoveMent( void )
{
#define SENSITIVE_RANGE 4
    int SRx1, SRy1, SRx2, SRy2;
    SRx1 = sX1 + SENSITIVE_RANGE;
    SRy1 = sY1 + SENSITIVE_RANGE;
    SRx2 = sX2 - SENSITIVE_RANGE;
    SRy2 = sY2 - SENSITIVE_RANGE;
    char _FAR_ *ptr = 0;
    int Move = 0;
    if ( MouseInRect( sX1, SRy1, SRx1, SRy2 ) ) {  // Left
        ptr = LeftCur;
        Move = MOVE_LEFT;
    } else if ( MouseInRect( SRx2, SRy1, sX2, SRy2 )) {  // Right
        ptr = RightCur;
        Move = MOVE_RIGHT;
    } else if ( MouseInRect( SRx1, sY1, SRx2, SRy1 )) {  // UP
        ptr = UpCur;
        Move = MOVE_UP;
    } else if ( MouseInRect( SRx1, SRy2, SRx2, sY2)) {   // Down
        ptr = DownCur;
        Move = MOVE_DOWN;
    }
    m_HideMouse;
    if ( ptr )
        x_define_mouse_cursor( ptr, MouseCursorColor );
    else
//andout	x_define_def_mouse_cursor( MouseCursorColor );
	x_define_mouse_cursor( defCursor, MouseCursorColor );
    m_UpdateMouse;
    return Move;
}

void _pascal_ zSprView::SetCanvas( zCanvas &c )
{
    myCanvas = &c;
    c.Viewer = this;
    c.PixSize = PixSize;
    c.HalfPixSize = PixSize/2;
    c.SaveUndo();
}

void _pascal_ zSprView::SelectViewer( BYTE mouBut )
{
    int i = 0;

    if ( mouBut == LCLICK ) {
        if ( RightViewIndex == Idx )
            return;
        LeftID = IndicObj->CGetID();
        SprView[LeftViewIndex]->myCanvas = 0;
        LeftViewIndex = Idx;
    } else if ( mouBut == RCLICK ) {
        if ( LeftViewIndex == Idx )
            return;
        RightID = IndicObj->CGetID();
        SprView[RightViewIndex]->myCanvas = 0;
        RightViewIndex = Idx;
        i = 1;
    } else
        return;

    SetCanvas( *Canvas[i] );
    Canvas[i]->AdjustPixSize();
    mmgr.CSendMsg( Canvas[i], REDRAW );
    for( i = 0; i < 4; i ++ )
        mmgr.CSendMsg( SprView[i]->IndicObj, REDRAW );
}

void _pascal_ zSprView::RedrawViewer( void )
{
    m_HideMouse;

    CClrContext();
    CopyToScreen();
    OutLineSprite();

    m_ShowMouse;
}

void _pascal_ zSprView::RedrawVC( void )
{
    mmgr.CSendMsg( this, REDRAW );
    mmgr.CSendMsg( myCanvas, REDRAW );
}

void _pascal_ zSprView::OutLineSprite( int color )
{
    v_Rectangle( sX1, sY1, OrigX+Width, OrigY+Height, color );
}

static void _pascal_ CopyLineY( int from, int to, int line1, int line2 )
{
    int i, j;
    VideoPage = VisiblePageOffs;
    for( i = from, j = 0; i <= to; i++, j++ )
        buffer[j] = (char)v_GetPixel( i, line1 );
    VideoPage = HiddenPageOffs;
    for( i = from, j = 0; i <= to; i++, j++ )
        v_PutPixel( i, line2, buffer[j] );
}

static void _pascal_ CopyLineX( int from, int to, int line1, int line2 )
{
    int i, j;
    VideoPage = VisiblePageOffs;
    for( i = from, j = 0; i <= to; i++, j++ )
        buffer[j] = (char)v_GetPixel( line1, i );
    VideoPage = HiddenPageOffs;
    for( i = from, j = 0; i <= to; i++, j++ )
        v_PutPixel( line2, i, buffer[j] );
}

void _pascal_ zSprView::DoMoveMent( BYTE mouBut, int Move )
{
    static Saved = 0;
    if ( mouBut == LHOLD ) {
        if ( friction.peekTime() >= 1 )
            friction.start();
        else
            return;
    } else if ( mouBut != RCLICK ) {
        Saved = 0;
        return;
    }

    if ( !PBMByteWidth(data))    return;

    int x2 = sX1 + Width, y2 = sY1 + Height;

    m_HideMouse;
    VideoPage = HiddenPageOffs;
    v_RectFill( OrigX, OrigY, x2, y2, v_cZero );
    switch( Move ) {
        case MOVE_UP:
            v_Putbmp( OrigX, sY1, data );
            CopyLineY( OrigX, x2, OrigY, y2 );
            break;
        case MOVE_DOWN:
            v_Putbmp( OrigX, OrigY+1, data );
            CopyLineY( OrigX, x2, y2, OrigY );
            break;
        case MOVE_LEFT:
            v_Putbmp( sX1, OrigY, data );
            CopyLineX( OrigY, y2, OrigX, x2 );
            break;
        case MOVE_RIGHT:
            v_Putbmp( OrigX+1, OrigY, data );
            CopyLineX( OrigY, y2, x2, OrigX);
            break;
        default:
            Saved = 0;
            return;
    }
    CopyToBuffer();
    VideoPage = VisiblePageOffs;
    CopyToScreen();

    if ( myCanvas && !Saved ) {
        Saved = 1;
        myCanvas->SaveUndo();
    }
    m_ShowMouse;
}

void _pascal_ zSprView::CopyToBuffer( void )
{
    v_Getbmp( OrigX, OrigY, Width, Height, data );
}

void _pascal_ zSprView::CopyToScreen( void )
{
    if ( PBMWidth(data))
        v_Putbmp( OrigX, OrigY, data );
}

void _pascal_ zSprView::SetWH( int w, int h )
{
    m_HideMouse;

    VideoPage = HiddenPageOffs;
    CClrContext();
    CopyToScreen();
    Width = w, Height = h;
    CopyToBuffer();
    VideoPage = VisiblePageOffs;

    CDrawOutLine();  // outlines the viewer
    CClrContext();
    CopyToScreen();
    if ( myCanvas ) {
        myCanvas->AdjustPixSize();
        mmgr.CSendMsg( myCanvas, REDRAW );
    }
    OutLineSprite( v_cWhite );

    m_ShowMouse;
}

void _pascal_ zSprView::SetNewBuf( char _FAR_ *buf )
{
    if ( buf ) {
        size_t size = (PBMWidth(buf)*PBMHeight(buf)+2);
        memcpy( data, buf, size );
    }
    SetWH( PBMWidth(data), PBMHeight(data) );
}

static void _pascal_ FlipString( char _FAR_ *str, int len )
{
    char _FAR_ *strEnd = str+len-1;
    char c;
    for ( int i = 0; i < len/2; i ++, str++, strEnd-- ) {
        c = *str;
        *str = *strEnd;
        *strEnd = c;
    }
}

void _pascal_ zSprView::FlipHorizontal( void )
{
    if ( !PBMByteWidth( data )) return;
    x_pbm_to_bm( data, DataConvert );
    int h = LBMHeight( DataConvert );
    int w = LBMWidth( DataConvert );
    char _FAR_ *bmp = DataConvert + 2;
    for( int i = 0; i < h; i ++, bmp += w )
        FlipString( bmp, w );
    x_bm_to_pbm( DataConvert, data );
    RedrawVC();
}

void _pascal_ zSprView::FlipVertical( void )
{
    if ( !PBMByteWidth( data )) return;
    x_pbm_to_bm( data, DataConvert );
    int h = LBMHeight( DataConvert );
    int w = LBMWidth( DataConvert );
    char _FAR_ *bmpBeg = DataConvert + 2;
    char _FAR_ *bmpEnd = DataConvert + 2 + ( w * (h-1) );
    for ( int i = 0; i < h/2; i ++, bmpBeg += w, bmpEnd -= w ) {
        memcpy( buffer, bmpBeg, w );
        memcpy( bmpBeg, bmpEnd, w );
        memcpy( bmpEnd, buffer, w );
    }
    x_bm_to_pbm( DataConvert, data );
    RedrawVC();
}

void _pascal_ zSprView::RotateLeft( void )
{
    int i, j, w, h, woh;
    if ( !PBMByteWidth( data )) return;
    x_pbm_to_bm( data, DataConvert );
    h = LBMHeight( DataConvert );
    w = LBMWidth( DataConvert );
    LBMHeight( CanvasData ) = h;
    LBMWidth( CanvasData ) = w;
    char _FAR_ *src = DataConvert + 2 + w - 1;
    char _FAR_ *dest = CanvasData + 2;
    char _FAR_ *destr = dest;
    char _FAR_ *srcr = src;
    if ( w > h )
       woh = h;
    else
        woh = w;
    for ( i = 0; i < woh; i ++ ) {
        for( j = 0; j < woh; j ++ ) {
            *destr = *srcr;
            srcr --;
            destr += w;
        }
        src += w;
        srcr = src;
        dest ++;
        destr = dest;
    }
    x_bm_to_pbm( CanvasData, data );
    RedrawVC();
}

void _pascal_ zSprView::RotateRight( void )
{
    int i, j, w, h, woh;
    if ( !PBMByteWidth( data )) return;
    x_pbm_to_bm( data, DataConvert );
    h = LBMHeight( DataConvert );
    w = LBMWidth( DataConvert );
    LBMHeight( CanvasData ) = h;
    LBMWidth( CanvasData ) = w;
    char _FAR_ *src = DataConvert + 2;
    char _FAR_ *dest = CanvasData + 2 + w - 1;
    char _FAR_ *destr = dest;
    char _FAR_ *srcr = src;
    if ( w > h )
       woh = h;
    else
        woh = w;
    for ( i = 0; i < woh; i ++ ) {
        for( j = 0; j < woh; j ++ ) {
            *destr = *srcr;
            srcr ++;
            destr += w;
        }
        src += w;
        srcr = src;
        dest --;
        destr = dest;
    }
    x_bm_to_pbm( CanvasData, data );
    RedrawVC();
}

DeclareObjH( SpriteView )
{
    static int Move, ChangeMove;
    zSprView &t = *((zSprView*)msg->ToObj);

    switch( msg->actId ) {
        case REDRAW:
            t.RedrawViewer();
            return 0;
        case MOUSE_ON:
            ChangeMove = 1;
            break;
        case CANCEL_CLICK:
        case MOUSE_OFF:
            if ( Move ) {
//andout		x_define_def_mouse_cursor( MouseCursorColor );
		x_define_mouse_cursor( defCursor, MouseCursorColor );
		ChangeMove = 1;
                Move = 0;
            }
            return 0;
        case 0:
            break;
        default:
            return 0;
    }

    if ( ChangeMove ) {
        Move = t.CheckMoveMent();
        ChangeMove = 0;
    }

    if ( Move ) {
        t.DoMoveMent( msg->mouButton, Move );
        if ( msg->mouButton & CLICK )
            mmgr.CSendMsg( t.myCanvas, REDRAW );
    } else
        t.SelectViewer( msg->mouButton );
    return 0;
}
