#ifndef _ViewMgr_H_
#define _ViewMgr_H_
#include "classes.h"
#include "macros.h"
#include "object.h"
#include "manager.h"
#include "timer.h"
#include "debug.h"

//
// Non-class functions and global variables
//
// save the mouse pos when it's pressed over an object.  When an obj
// handler is invoked by mouse.  The position of the mouse when it was
// pressed is in these two vars.  Unless mouse is out of existence, these
// global variables will not change!
//
extern int objMouseX;
extern int objMouseY;
extern int help;

#define TRACE_MOUSE

class ViewMgr:public ScrObj, public ObjMgr {
protected:
	BBOOL         slave;
	static BYTE ButtonStatus,	// Current mouse button status
                ButtonDown;     // Current button holding down
    static BBOOL objFocus;      // Can I change current object?
	static Timer downTimer,		// Count how long a button's been held down
				 DCTimer;		// Count delay between two clicks to determine
				 				// if it's a double click.
    static BBOOL ContinuousHold;
    static  ScrObj  *WorkObj;   // Current object to work on. Can't be change
                                // if objFocus is on.
    static Message *otoSender,  // use to send message from object to object
				   *otoTail;
    static int nClickCount;

    // kill current message and get next one
    static void  _pascal_ AdvanceMsg( void );
    virtual void  _pascal_ ObjHandler(void );
    virtual void  _pascal_ KeyBoardHandler( void );
    virtual void  _pascal_ MouseHandler( void );

public:
    static void  _pascal_ CSetHoldMsgCnt( BBOOL v ) { ContinuousHold = v; }
    static void  _pascal_ CResetObjectHandler ( void );
    static void  _pascal_ CMakeMouseOnNoObject( void );
    static void  _pascal_ CSetObjectFocus     ( BBOOL focus )
        { objFocus = focus; }
    static void  _pascal_ CToggleObjectFocus  ( void );
    static void  _pascal_ CResetClicks        (void )
        { nClickCount = 0;  }
	// Save clicks to simulate multi clicks
    static int  _pascal_ CAddClick  (const Message *msg );
    static void _pascal_ CSendMsg   (Object *tg, int id, Object *src = 0,
                                    BYTE _mb = 0, int key = 0 );
    static void _pascal_ CLinkSendMsg (Object *tg, int id,Object *src = 0,
                                    BYTE _mb = 0, int key = 0);
    void     _pascal_ CShowAllObject( void );

    static void _pascal_
    CSendMsg2Object( int id, Object *src = 0, BYTE _mb = 0, int key = 0 )
    {
        CSendMsg( WorkObj, id, src, _mb, key );
    }
#ifdef TRACE_MOUSE
    virtual void     _pascal_ CheckMouseTrace( void );
    static  BBOOL   TraceMouse;
#endif

    _pascal_ ViewMgr(int x1, int y1, int x2, int y2, Object *object = 0,
                    OBJH handler = 0, WORD id = 0, BBOOL isSlave = 0 )
            :ObjMgr(object ),
             ScrObj(x1, y1, x2, y2, handler, id ),
             slave(isSlave)                         { }
    _pascal_ ViewMgr( void ):ObjMgr(), ScrObj(), slave(0)     { }
    _pascal_ ~ViewMgr()                                       { }

    virtual int      _pascal_ CMgrLoop        ( void );
    virtual void     _pascal_ CShowObject( void )             { }
    ViewMgr&         _pascal_ operator= (const ViewMgr &y);
    BBOOL            _pascal_ CIsSlave  ( void )              { return slave; }
    virtual  BBOOL   _pascal_ CGiveUpControl ( void )         { return TRUE; }
    BBOOL            _pascal_ VerifyObject( void );
    virtual  int     _pascal_ identify( void )              { return VIEWMGR_ID; }
    void  _pascal_ CReset( void );

}; // end of class ViewMgr

#endif

/* Source for CheckMouseTrace:

static void _pascal_ ShowBorder( Object* obj, BBOOL draw = TRUE )
{
	m_HideMouse;
    if ( draw ) ((ScrObj*)obj)->CDrawOutLine( 92 );
    else switch( obj->identify()) {
    case BUTTON_ID:
        ((Button*)obj)->CDraw3DOutLine();
        break;
    default:
        ((ScrObj*)obj)->CDrawOutLine();
    }
    m_ShowMouse;
}

#ifdef TRACE_MOUSE
void ViewMgr::CheckMouseTrace()
{
    static int oldMx, oldMy;
    static int newMx, newMy;
    static ScrObj *dummy;
	oldMx = newMx;
	oldMy = newMy;
	newMx = m_MouseX;
	newMy = m_MouseY;
	if (oldMx != newMx || oldMy != newMy ) {
		objMouseX = newMx;
		objMouseY = newMy;
        dummy = 0;
        if ( Current ) dummy = WorkObj;
        if ( VerifyObject() ) {
            ViewMgr::CSendMsg( WorkObj, MOUSE_ON, 0, 0 );
            ShowBorder( WorkObj );
        }
        if ( dummy && dummy != WorkObj ) {
            ViewMgr::CSendMsg( dummy, MOUSE_OFF, 0, 0 );
            ShowBorder( dummy, FALSE );
        }
    }
}
#endif
*/
