#include "sprite.h"
#include "vidinter.h"
#include "timer.h"
#include <conio.h>
#include <stdlib.h>
#include <stdio.h>
#include <DOS.h>
#include "debug.h"
#include "fileio.h"
//#include <xpal.h>
#include "xpal.h"
#include "xpbitmap.h"

#ifdef NormalTimer
#error Need fast timer turned on.
#endif

//
// bitmap member functions
//

// bitmap::CPut
//     Draw the bitmap to screen page ScrnOffs at pos x, y
void bitmap::CPut( WORD x, WORD y, WORD ScrnOffs )
{
    x_put_pbm( x, y, ScrnOffs, data );
}

// bitmap::CPutMasked
//     Same as bitmap::CPut except black colors are transparent
void bitmap::CPutMasked( WORD x, WORD y, WORD ScrnOffs )
{
    x_put_masked_pbm( x, y, ScrnOffs, data );
}


void bitmap::CGet( WORD x, WORD y, BYTE width, BYTE height,
              WORD ScrnOffs )
{
    WORD not_even = width % 4;
    if ( not_even )
        width += 4 - not_even;

    WORD datasize = height * width + 2;
    if ( data ) {
        if ( mysize != datasize) {
            delete data;
            data = new BYTE [datasize];
            mysize = datasize;
        }
    } else {
        data = new BYTE [datasize];
        mysize = datasize;
    }

    x_get_pbm( x, y, width/4, height, ScrnOffs, data );
}

void bitmap::CSetData( BYTE far *idata )
{
    if ( !idata ) {
       printf("Error: bitmap::CSetData   in data (idata) is null.\n");
       return ;
    }
    if ( data )
                delete data;
    data = idata;
}

bitmap& bitmap::operator=( const bitmap &y )
{
    if (this != &y) {
        (*this).data = y.data;
        (*this).next = y.next;
    }
    return *this;
}

void bitmap::CSize( int orig, int dest )
{
    if (orig == dest) return;
   printf("sizing bitmap from size %d to size %d\n", orig, dest );
}

int bitmap::CLoad( char *filename )
{
    if (!filename) {
       printf("Error: bitmap::CLoad  null filename.\n");
       return 0;
    }
   printf("load bitmap file %s.\n", filename);
   return 0;
}

//
// sprite memeber functions
//

bitmap* sprite::CAdvance( WORD ScrnOffs )
{
    if ( !current )
        current = firstbmp;

    current->CPutMasked( x, y, ScrnOffs );
   current = current->next;

    if ( !current ) {
       printf("Error: sprite::CAdvance   result current is null.");
    }

    return current;
}


void   sprite::CRefresh( WORD ScrnOffs )
{
    if (!current) return;
    current->CPutMasked( x, y, ScrnOffs );
}

sprite::~sprite ()
{
   bitmap *tmpPtr;

    if (firstbmp) {

        lastbmp->next = 0;

        while (firstbmp) {

            tmpPtr = firstbmp;
            firstbmp = firstbmp->next;

            delete tmpPtr;
        }
    }
}

bitmap* sprite::CAdd( bitmap *newbmp )
{
    bitmap *tmpPtr;

    if (newbmp)
               tmpPtr = newbmp;
    else
                tmpPtr = new bitmap();

    if ( !tmpPtr ) {
       printf("Error:  sprite::CAdd   tmpPtr is null.\n");
    }

    if (!firstbmp)
        current = firstbmp = lastbmp = tmpPtr;

    else {

        lastbmp->next = tmpPtr;
        lastbmp = tmpPtr;

    }

    lastbmp->next = firstbmp;
    return tmpPtr;
}

sprite& sprite::operator=( const sprite &spr )
{
    if (this != &spr) {
        (*this).firstbmp = spr.firstbmp;
        (*this).lastbmp = spr.lastbmp;
        (*this).current = spr.current;
        (*this).x = spr.x;
        (*this).y = spr.y;
        (*this).width = spr.width;
        (*this).height = spr.height;
    }
    return *this;
}

/*
BYTE far* readfile(char *filenamein ){
    BYTE far *ibuff = NULL;
    unsigned infilesize, nread;
    FILE *fi;

   fi = fopen( filenamein,"rb+" );
   if ( !fi ) return 0;

   fseek ( fi, 0, SEEK_END );
   infilesize = (unsigned) ftell(fi);
   fseek ( fi, 0, SEEK_SET );

    ibuff = new BYTE far[infilesize];
    if ( !ibuff ) return 0;

    int fih = fileno( fi );
    _dos_read( fih, ibuff, infilesize, &nread );

    if ferror( fi ) return 0;

    fclose(fi);
    return ibuff;
}

*/

#define SPRITE_WIDTH 24

#ifndef NormalTimer
#define LOOP_DELAY 8
#else
#define LOOP_DELAY 1
#endif

int main ( int argc, char **argv)
{
    int i, j, tiles = 90;
   int loopDelay = LOOP_DELAY;

    char *temp_buffer = new char [150 * 240] ;

    if ( !temp_buffer ) {
        printf("Error!  Allocate %d bytes of memory failed.\n", 150 * 240 );
        exit (0);
    }

   if ( argc > 1 )
      tiles = atoi( argv[1] );
   if ( argc > 2 )
      loopDelay = atoi( argv[2] );

   printf ( "tiles: %d  loopDelay: %d\n", tiles, loopDelay);

    int slow_advance = 3, fast_advance = 6, block_size = 24, FrameRateTick=12;

    if ( argc > 3 )
        slow_advance = atoi( argv[3]) ;
    if ( argc > 4 )
        fast_advance = atoi( argv[4]) ;
    if ( argc > 5 )
        block_size = atoi( argv[5] ) ;
    if ( argc > 6 )
        FrameRateTick = atoi( argv[6] ) ;

    int divisor = (int) (block_size / slow_advance);

    int tiles_div4 = (int) ( tiles / divisor );
    int m1 = 0, m2 = tiles_div4;

    printf("slow_advance: %d  divisor: %d  tiles_div4: %d\n",
            slow_advance, divisor, tiles_div4 );

   while(!kbhit());

   if (!getch()) getch();

   _InitVideo;

//
// Read data from disks
//
// pallettes
//
   BYTE far *wmpal = readfile( "data\\walkman.pal");
    if (!wmpal) {
       printf("Main: wmpal is null.\n");
    }
   BYTE far *stdpal = readfile( "data\\standard.pal" );
    if (!stdpal) {
       printf("Main: stdpal is null.\n");
    }
   x_put_pal_raw( wmpal, 256, 0 );
//
// Picture contains walkman sprites
//
   x_put_masked_pbm( 0, 0, VisiblePageOffs,
               readfile( "data\\walkman.pbm" ) );
   bitmap manface[4];
   sprite walkman, wmCmp;
   walkman.CMoveto( 0, 100 );   // init the sprite position
   walkman.CSetWH( 24, 24 );
   wmCmp.CMoveto( 0, 81 );
   wmCmp.CSetWH( 24, 24 );

    for(i=0; i<3; i++)           // Grab the bitmaps
      manface[i].CGet( i * SPRITE_WIDTH + 4 * 24, 0, 24, 24 );

   for(i=0; i<4; i++) {         // Set the sprite's bitmaps
      walkman.CAdd( &(manface[i]) );
      wmCmp.CAdd( &(manface[i]));
   }
//
// Circle sprites
//
   bitmap cir[24];
   sprite circle;
   circle.CSetWH( 32, 32 );
   circle.CMoveto( 155, 100 );
   char name[20];
   for (i = 1; i < 13; i ++ ) {
      sprintf(name, "data\\cir%d.pbm", i);
      cir[i-1].CSetData( readfile( name ));
      circle.CAdd( &cir[i-1] );
   }
   for ( i=12, j=11; i < 24; i++, j--) {
        cir[i] = cir[j];
        circle.CAdd(&cir[i]);
    }
//
// Wizard
//
    bitmap wiz1;
   wiz1.CSetData( readfile( "data\\boodwiz.pbm" ) );
//
// Grumjug
//
   sprite gj;
   gj.CMoveto( 160, 120 );
   gj.CSetWH( 24, 20 );
   bitmap gjpic[5];
   for (i = 0; i < 5; i++ ) {
      sprintf(name, "data\\grumjug%d.pbm", i);
      gjpic[i].CSetData( readfile( name ));
      gj.CAdd( &gjpic[i] );
   }
   sprite gjStation = gj;
   gjStation.CMoveto( 10, 10 );

///////////  End of Disk Reading ///////////////

//
// Set up the screen
//
   x_rect_fill(0, 0, ScrnPhysicalPixelWidth,v_SplitScrnHeight,SplitScrnOffs,5);
   x_line(0,0,ScrnPhysicalPixelWidth,0,2,SplitScrnOffs);

   x_show_splitscreen();

   for (i=ScrnPhysicalHeight;i>ScrnPhysicalHeight-v_SplitScrnHeight;i--){
      x_adjust_splitscreen(i);
    }

    x_rect_fill( 0, 0, ScrnLogicalPixelWidth, ScrnLogicalHeight,
                 VisiblePageOffs, 252 );
    x_rect_fill( 0, 0, ScrnLogicalPixelWidth, ScrnLogicalHeight,
                 HiddenPageOffs, 252 );
    x_put_pal_raw( stdpal, 256, 0 );     // set pal

    x_line(ScrnLogicalPixelWidth,0,ScrnPhysicalPixelWidth,0,28,VisiblePageOffs);
    x_line(ScrnLogicalPixelWidth,0,ScrnPhysicalPixelWidth,0,28,HiddenPageOffs);
    x_line(ScrnLogicalPixelWidth,1,ScrnPhysicalPixelWidth,1,43,VisiblePageOffs);
    x_line(ScrnLogicalPixelWidth,1,ScrnPhysicalPixelWidth,1,43,HiddenPageOffs);
    x_line(0,0,ScrnPhysicalPixelWidth,0,43,VisiblePageOffs);
    x_line(0,0,ScrnPhysicalPixelWidth,0,43,HiddenPageOffs);
    x_line(0,1,ScrnPhysicalPixelWidth,1,28,VisiblePageOffs);
    x_line(0,1,ScrnPhysicalPixelWidth,1,28,HiddenPageOffs);
//    x_hide_splitscreen();
//
// Draw the wizard sprite
//
    wiz1.CPut( 10 , 10 );
    wiz1.CPut( 10 , 10, HiddenPageOffs );
    while(!kbhit());
    if (!getch()) getch();
//
// Draw the walkman sprites
//
    manface[3] = manface[1];
    for(i=0; i<4; i++)  {   // display the bitmaps
        manface[i].CPut( i * 24, 50 );
        manface[i].CPut( i * 24, 50, HiddenPageOffs );
    }

//
// Draw the color bars
//
   BYTE color = 0;
    for (i=0; i<ScrnLogicalPixelWidth; i++) {
        for ( j = 160; j < ScrnLogicalHeight; j++ ) {
           x_put_pix( i, j, VisiblePageOffs, color );
           x_put_pix( i, j, HiddenPageOffs, color );
        }
        color ++;
    }
/////////// End of Screen setup ////////////////

//
// Set up for saving backgrounds the sprites erased
//
    bitmap backdata, bdatawm, bdatawmCmp;
    backdata.CGet( circle.sx(), circle.sy(),
                   circle.w(), 40);
    bdatawm.CGet( walkman.sx(), walkman.sy(),
                    walkman.w(), walkman.h());
    bdatawmCmp.CGet( wmCmp.sx(), wmCmp.sy(),
                    wmCmp.w(), wmCmp.h());
    WORD ox = circle.sx(), oy = circle.sy();
    WORD owx = walkman.sx(), owy = walkman.sy();
    WORD owcx = wmCmp.sx(), owcy = wmCmp.sy();

//
// Init interrupted timer (if used)
//
#ifndef NormalTimer
   #define GJ_SLOW_DELAY 11
   #define GJ_FAST_DELAY 11
    #define MAN_DELAY 12
    Timer::CInstallTimer( 8, 8192 );
#else
   #define GJ_SLOW_DELAY 2
   #define GJ_FAST_DELAY 1
   #define MAN_DELAY 3
#endif
//
// Set up timed animations
//
   Timer advanceDelay, wmDelay, Delay;
   Timer FrameRateCtrl;
    BYTE quit = 0;
    int scroll = 0, advance = 1, AdvanceD = GJ_SLOW_DELAY, menAdv = 0;
//
// Init first position
//
    wmCmp.CRefresh( HiddenPageOffs );
   walkman.CRefresh( HiddenPageOffs );
    i = 180;
    j = 180;

//    int refresh = 0;

//
// And here goes the animation loop
//
    advanceDelay.start();
    wmDelay.start();
   FrameRateCtrl.start();

    while ( !quit ) {

// now animate the sprite

    // It's time to move the two walking men

      if ( wmDelay.peekTime() >= MAN_DELAY ) {

        // advance to next bitmap in the sprite sequence

            wmCmp.CAdvance( HiddenPageOffs );
            walkman.CAdvance( HiddenPageOffs );

        // restart timer

            wmDelay.start();

        // set advance flag to true

         menAdv = 1;

    // otherwise we need to refresh them

      } else {

         wmCmp.CRefresh( HiddenPageOffs );
         walkman.CRefresh( HiddenPageOffs );

      }

// and draw the new sprite to hidden page

    // It's time to move the screen and our main character(grumjug)
      if ( advanceDelay.peekTime() >= AdvanceD && advance) {

    // advance to next bitmap in the sprite sequence

            circle.CAdvance( HiddenPageOffs );
         gj.CAdvance( HiddenPageOffs );

    // restart timer

         advanceDelay.start();

    // otherwise we need to refresh the character(so he won't get erased)
      } else {

         circle.CRefresh( HiddenPageOffs );
         gj.CRefresh( HiddenPageOffs );

      }

// Show the new screen
// Waste some time by drawing the same bitmap for a number of times

         if ( m2 > tiles ) {
             m1 = 0;
             m2 = tiles_div4;
         }
         for ( int k = m1; k < m2; k++ )
             gjStation.CAdvance( SplitScrnOffs );
         m1 = m2;
         m2 += tiles_div4;

        while( FrameRateCtrl.peekTime() < FrameRateTick );
        x_page_flip( scroll, 1 );

      while(StartAddressFlag);
        FrameRateCtrl.start();

// wait a while
/*
        Delay.start();
      while( Delay.peekTime() < loopDelay );
*/


// Move the visual screen a little to the left

        scroll += advance;

        if ( scroll >= ScrnLogicalPixelWidth ) {
            scroll = 0;
      } else if ( scroll < 0 ) {
            scroll = ScrnLogicalPixelWidth - 1;
        }

// then erase the old sprite on hidden page

      backdata.CPut( ox, oy, HiddenPageOffs);
      bdatawm.CPut( owx, owy, HiddenPageOffs );
      bdatawmCmp.CPut( owcx, owcy, HiddenPageOffs );

// and save the new position

      ox = circle.sx();
      oy = circle.sy();
      owx = walkman.sx();
      owy = walkman.sy();
      owcx = wmCmp.sx();
      owcy = wmCmp.sy();

// also save the background at the new positions

      backdata.CGet( ox, oy, circle.w(), 40, HiddenPageOffs );

      bdatawm.CGet( owx, owy, walkman.w(), walkman.h(),
                  HiddenPageOffs );

      bdatawmCmp.CGet( owcx, owcy, walkman.w(), walkman.h(),
                  HiddenPageOffs );

// Set the new position of the circle and grumjug, so they will be at the
// center of the physical screen.

      circle.CMoveto( scroll + 155, 100 );
      gj.CMoveto( scroll + 160, 120 );

// Check if should move the two little men

      if ( menAdv ) {

         menAdv = 0;

            j += slow_advance - 1;
            i += slow_advance - 2;

         walkman.CMoveto( j, 105 );
         wmCmp.CMoveto( i, 81 );

            if ( j >= ScrnLogicalPixelWidth )
                j = 0;
            if ( i >= ScrnLogicalPixelWidth )
                i = 0;
      }

// Check for key pressed

      if ( kbhit() ) {
         int key;
            if ( !(key = getch()) )
                key = getch();
         switch( key ) {
            case 27:
               quit = 1;
               break;
            case 75:
//                    advance = -slow_advance;
//                    AdvanceD = GJ_SLOW_DELAY;
               break;
            case 77:
                    advance = slow_advance;
               AdvanceD = GJ_SLOW_DELAY;
               break;
            case 115:
//                    advance = -fast_advance;
//                    AdvanceD = GJ_FAST_DELAY;
               break;
            case 116:
                    advance = fast_advance;
               AdvanceD = GJ_FAST_DELAY;
               break;
            default:
               gj.CReset();
               circle.CReset();
               advance = 0;
         }
      }
   }

// Remove splited screen

    x_show_splitscreen();
   for (i=ScrnPhysicalHeight-v_SplitScrnHeight;i<=ScrnPhysicalHeight;i++){
        x_adjust_splitscreen(i);
    }
// Remove timer

    Timer::CRemoveTimer();
    _RestoreVideo;

    return (0);
}
