#include "timer.h"


//
// Start
//

void _pascal_ Timer::start( void )
{
#ifdef __MSDOS__

#ifndef NormalTimer
    startTime = ticks;
#else
    static unsigned curTime;
    asm {
        mov AH, 0;
        int 1AH;
        mov curTime, DX;
    }
    startTime = curTime;
#endif

#endif

#ifdef __LINUX__
	startTime = 0;
#endif
}

//
// Stop
//

void _pascal_ Timer::stop( void )
{
    static unsigned endTime;
#ifdef __MSDOS__

#ifndef NormalTimer
    endTime = ticks;
#else
    asm {
        mov AH, 0;
        int 1AH;
        mov endTime, DX;
	}
#endif

#endif

#ifdef __LINUX__
	endTime = 50;
#endif

    if ( startTime <= endTime )
        elapsedTime = endTime - startTime;
    else
        elapsedTime = ( endTime + (0xffff - startTime));
}

//
// Peektime
//

unsigned _pascal_ Timer::peekTime( void )
{
    static unsigned endTime;
#ifdef __MSDOS__

#ifndef NormalTimer
    endTime = ticks;
#else
    asm {
        mov AH, 0;
        int 1AH;
        mov endTime, DX;
	}
#endif

#endif

#ifdef __LINUX__
	endTime = 50;
#endif

    if ( startTime <= endTime )
        return ( endTime - startTime );
    else
        return ( endTime + (0xffff - startTime));
}

//
// Timer interrupt handling
//

#ifndef NormalTimer

unsigned Timer::ticks = 1;
unsigned Timer::newDivisor = 1;

DeclareOldIsr( oldTimerIsr );
DeclareNewIsr( newTimerIsr )
{
    static unsigned Counter;

    Timer::ticks ++;
    Counter ++;

    if ( Counter == Timer::newDivisor ) {
        Counter = 0;
        oldTimerIsr ();
    } else {
        asm {
            mov AL, 20h
            out 20H, AL
        }
    }
}

//
// Change speed of timer
//

void _pascal_ Timer::ChangeSpeed( unsigned newDiv, unsigned newSpeed )
{
    static IsrInstalled;
    if ( newDiv != 1 ) { // 0 => restore    > 1 => change speed
        asm {
            CLI         // Interrupts off while resetting
            MOV AL,00110110B    // 00xx xxxx = channel 0
                        // xx11 xxxx = write LSB then MSM
                        // xxxx 011x = mode 3
                        // xxxx xxx0 = binary system
            OUT 43H,AL
            MOV BX,newSpeed // New divisor
            MOV AL,BL
            OUT 40H,AL      // Send LSB
            MOV AL,BH
            OUT 40H,AL      // Send MSB
            STI
        }
    }
    if ( !IsrInstalled ) {
        IsrInstalled = 1;
        installisr( oldTimerIsr, newTimerIsr, 0x08 );
    }
    if ( !newDiv ) {
        IsrInstalled = 0;
        restoreisr( 0x08, oldTimerIsr ); // remove interrupt handler
    } else
        newDivisor = newDiv;
}

#endif // routines required for fast timer

//
// Test timer
//

//#define TEST_TIMER
#ifdef TEST_TIMER

#include <stdio.h>
#include <dos.h>
#include <stdlib.h>
void main ( void )
{
#ifdef NormalTimer
	#error Required fast timer to be turned on.
#endif
    if (!Timer::CInstallTimer(10, 6555)) {
		printf("Required fast timer.\n");  // secondary protections. :)
        exit(0);
    }
	Timer w;
	int i;
	for (i = 550; i <= 10000; i += 550 ) {
		w.start();
		delay( i );
		w.stop();
		printf("delay: %4d ms  time: %3d\n", i, w.time() );
	}
	Timer::CRemoveTimer();
}

#endif
