#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "inifile.h"

int _pascal_ zINIFile::FindString( char *str, int item )
{
    char *tmp;
    int l = strlen( str );
    while( 1 ) {
        tmp = fgets( buf, 80, fp );
        if ( !tmp ) break;
        if ( item && tmp[0] == '[') break;
        if ( tmp[0] == str[0] && strncmp(tmp, str, l ) == 0 ) {
            tmp[strlen(tmp)-1] = 0;
            return 1;
        }
    }
    Error = INI_NOT_FOUND_ERROR;
    return 0;
}

char * _pascal_ zINIFile::StringValue( char c )
{
    char *tmp = buf;

    while( *tmp && *tmp != '=' ) tmp++;

    if ( !(*tmp)) {
        Error = INI_SYNTAX_ERROR; // No '='
        return 0;
    } else while( *tmp ) {
        if ( *tmp == c ) {
            while( *(++tmp) && (*tmp == ' ' || *tmp == '\t' || *tmp == '\n' ))
            if ( *tmp )
                return tmp;
            Error = INI_NO_VALUE_ERROR;
            return 0;
        }
        tmp ++;
    }
    Error = INI_TYPE_ERROR;
    return 0;
}

int _pascal_ zINIFile::FindItem( char *ItemName, long int &Value )
{
    if ( fp ) {
        fseek( fp, SectionLock, SEEK_SET );
        if ( FindString( ItemName, 1 )) {
            char *tmp = StringValue( '#' );
            if ( tmp ) {
                if ( strcmpi( tmp, "on" ) == 0 )
                    Value = 1;
                else
                    Value = atol( tmp );
                return 1;
            }
        }
    } else
        Error = INI_NO_FILE_ERROR;

    return 0;
}

int _pascal_ zINIFile::FindItem( char *ItemName, char *Value )
{
    if ( fp ) {
        fseek( fp, SectionLock, SEEK_SET );
        if ( FindString( ItemName, 1 )) {
            char *tmp = StringValue( '$' );
            if ( tmp ) {
                strcpy( Value, tmp );
                return 1;
            }
        }
    } else
        Error = INI_NO_FILE_ERROR;
    return 0;
}

int _pascal_ zINIFile::FindSection( char *SectName )
{
    if ( fp ) {
        if ( buf ) {
            if ( FindString( SectName )) {
                SectionLock = ftell( fp );
                if ( SectionLock > -1 )
                    return 1;
                else
                    Error = INI_LOCK_ERROR;
            }
        } else
            Error = INI_NO_BUF_ERROR;
    } else
        Error = INI_NO_FILE_ERROR;
    return 0;
}


int _pascal_ zINIFile::Open( char *fn )
{
    Close();
    if ( fn ) {
        fp = fopen( fn, "r" );
    }
    if ( fp )
        return 1;
    else
        return 0;
}

int _pascal_ zINIFile::Close( void )
{
    if ( fp ) {
        if ( fclose( fp ) == 0) {
            fp = 0;
            return 1;
        }
    }
    return 0;
}


// #define TEST

#ifdef TEST
typedef struct zMark
{
    char *str;
} Mark;

main( int argc, char **argv )
{
    Mark strmark[] = { { "FaceColor" }, { "String" }, {"NullString"} };
    Mark intmark[] = { { "FaceColor2" }, { "tFaceColor" }, { "NoValue" } };
    if ( argc > 1 ) {
        zINIFile test;
        char buffer[512];

        test.SetWorkBuf( buffer );
        test.Open( argv[1] );
        if ( !test.FindSection( "[Test]" ) ) {
            printf("section not found.\n");
            return 1;
        }

        char data[128];

        int i;
        for ( i = 0; i < 3; i ++ ) {
            if ( test.FindItem( strmark[i].str, data ))
                printf("%s: %s\n", strmark[i].str, data );
            else
                printf("%s: %d  failed.\n", strmark[i].str, test.ErrorNo() );
        }

        printf("\n");

        long int d;

        for ( i = 0; i < 3; i ++ ) {
            if ( test.FindItem( intmark[i].str, d))
                printf("%s: %ld\n", intmark[i].str, d );
            else
                printf("%s: %d  failed.\n", intmark[i].str, test.ErrorNo() );
        }

    }
    return 0;
}
#endif
