#include "strbox.h"
#include "macros.h"
#include "viewmgr.h"
#include "keys.h"
#include "vidinter.h"
#include "stredit.h"

#ifdef __MSDOS__
#include <dos.h>
#endif

// Global Variable ( The text blinking caret )
Caret gTextCaret;
// End of Global Variable

int Caret       ::x             = 0;
int Caret       ::y             = 0;
int Caret       ::y2            = 0;
unsigned Caret  ::Delay         = 2;
Timer Caret     ::blink;
BBOOL Caret     ::on            = FALSE;
BBOOL Caret     ::draw          = FALSE;

void _pascal_ Caret::CDraw( void )
{
    draw = FALSE;
    if( gStrEditor.CIsInsert() )
        v_PutLine( x, y, x, y2, HighLiteColor );
    else
        v_PutLine( x, y2+1, x+4, y2+1, HighLiteColor );
}


void _pascal_ Caret::CErase( void )
{
    static BBOOL i = gStrEditor.CIsInsert();
    draw = TRUE;
    if ( i )
        v_PutLine( x, y, x, y2, FaceColor );
    else
        v_PutLine( x, y2+1, x+4, y2+1, FaceColor );
    i = gStrEditor.CIsInsert();

}


DeclareIntCClassNoPtr( Caret, ShowCaret )
{
    if ( blink.peekTime() >= Delay ) {

        if ( draw )
                    CDraw();
        else
                    CErase();
        blink.start();
	}

	return 0;
}

void _pascal_ Caret::CShow( void )
{
	if ( !on ) {

		CDraw();
		on = TRUE;

		blink.start();

        CurrentMgr->CRegInt( ShowCaret, 0, TRUE );

	}
}

void _pascal_ Caret::CHide( void )
{
	if ( on ) {

		CErase();
		on = FALSE;

        CurrentMgr->CUnRegInt( ShowCaret, TRUE );

	}
}

int StrBox  ::  PixelLen;
char *StrBox::  FrameSPtr;
int StrBox  ::  FPtrIndex;
int StrBox  ::  FrameIndex;
int StrBox  ::  textX1;

int _pascal_ StrBox::CRestoreOutLine( void )
{
    if ( Button::CRestoreOutLine() ) {
        CDrawOutLine();
        return OutLineColor;
    }

    return 0;
}

void _pascal_ StrBox::CShowObject( void )
{
    CHighLiteOutLine();
    CRestoreOutLine();

    CDrawFace();
    CDrawOutLine();

    m_ShowMouse;
    ViewMgr::CSendMsg( this, REDRAW );
}


int _pascal_ StrBox::CShowName( void )
{
    char *t = zsBname;
    if ( !t ) t = StrPtr;
    if ( t )
        return v_LimitPixelPrint( x1() + 2, CCenterMsgY(), TextColor,
                                FaceColor, x2() - x1() - 3, t );
    else
        return sX1+3;
}

void _pascal_ StrBox :: InitEdit( char *str )
{
    m_HideMouse;

    textX1      = x1() + 2;
    PixelLen    = x2() - textX1 - 2;

    str[0]      = 0;

    if ( StrPtr != zsBname ) {
        delete StrPtr;
        StrPtr = 0;
        if ( zsBname ) {
            StrPtr = new char[strlen(zsBname)+1];
            strcpy( StrPtr, zsBname );
        }
    }
    if ( StrPtr ) {
        strcpy( str, StrPtr );
        FrameIndex   = strlen( str );
        FrameSPtr    = v_StrToFitPixels( str, str + FrameIndex - 1,
                                        PixelLen, FPtrIndex );
        gTextCaret.CSetX( v_StrPixelLenUpTo (  FrameSPtr,
                        FrameIndex - FPtrIndex, PixelLen ) + textX1 );
        delete StrPtr;
        StrPtr = 0;
    } else  {
        FrameIndex   = FPtrIndex = 0;
        FrameSPtr    = str;
        gTextCaret.CSetX( textX1 );
	}
    zsBname = str;
    gStrEditor.CSetMaxLen( 256 );
    gStrEditor.CSetStr( str, FrameIndex );
	CHighLiteOutLine();
	gTextCaret.CSetY( CCenterMsgY() );
    v_LimitPixelPrint( textX1, CCenterMsgY ( ), TextColor,
                        FaceColor, PixelLen, FrameSPtr );
    gTextCaret.CShow ( );
    m_ShowMouse;
    ViewMgr::CSetObjectFocus( TRUE );
    ViewMgr::TraceMouse = FALSE;
}

static char *str = 0;   // memory use when editing string
static BBOOL receiving; // true if entering string
static StrBox *myPtr;

void _pascal_ StrBox::StopEdit( void )
{
    gTextCaret.CHide();
    if ( receiving ) {  // Make sure no second entrance
        receiving = FALSE;

        if ( gStrEditor.CLen() )
        {
            myPtr->StrPtr = new char[gStrEditor.CLen()+1];
            if( !myPtr->StrPtr )
                perror("new in strbox");
            else
                strcpy( myPtr->StrPtr, str );
        } else
            myPtr->StrPtr = 0;

        myPtr->zsBname = myPtr->StrPtr;

        ViewMgr::CSetObjectFocus( FALSE );
#ifdef TRACE_MOUSE
        ViewMgr::TraceMouse = TRUE;
#endif
        myPtr->CShowObject();  // will restore outline also
    }
}

void _pascal_ StrBox::UpdateBox( int result, const Message *msg )
{
    //
    //  Update string on screen accordingly
    //
    if ( result == ENTER ) {    // Enter pressed, done editing
        StopEdit();
    } else if ( gStrEditor.CIsUpdate() ) {
        BBOOL refreshMsg = TRUE;
        FrameIndex = gStrEditor.CPos();
        // frame at begin segment of the string
        if ( FrameIndex == 0 ) {    // HOME
            FrameSPtr = str;
            FPtrIndex = 0;
        // frame at last segment of the string
        } else if ( FrameIndex == gStrEditor.CLen() ) {  // END
            beep( 200, 1 );
            FrameSPtr = v_StrToFitPixels( str, str + FrameIndex -1,
                                     PixelLen, FPtrIndex );
        // frame move left
        } else if ( FrameIndex < FPtrIndex || msg->keyDown == BACKSPACE ) {
            if ( FrameSPtr > str ) {
                FrameSPtr  --;
                FPtrIndex  --;
            }
        // frame move right
        // ( framePixelLen and PixelLen must < v_Width )
        } else if ( v_StrPixelLenUpTo( FrameSPtr,
                    FrameIndex - FPtrIndex, v_Width ) > PixelLen ) {
            do {  // keep looping until enough pixel is freed up in the
                  // frame so the character to the right can FrameIndext in
                FrameSPtr  ++;
                FPtrIndex  ++;
            } while ( v_StrPixelLenUpTo( FrameSPtr,
                    FrameIndex - FPtrIndex, v_Width ) > PixelLen );
		} else if ( result )  // key didn't affect string content
            refreshMsg = FALSE;
        if ( myPtr->CMouseOnMe() )  m_HideMouse;
        gTextCaret.CErase();
        gTextCaret.CSetX( v_StrPixelLenUpTo( FrameSPtr,
                            FrameIndex - FPtrIndex, PixelLen ) + textX1 );
        if ( refreshMsg )
            v_LimitPixelPrint( textX1, myPtr->CCenterMsgY ( ),
                               TextColor, FaceColor, PixelLen, FrameSPtr );
        gTextCaret.CDraw();
        m_ShowMouse;
    }
}

DeclareObjHClass( StrBox, CStrBoxHand )
{
    myPtr = (StrBox*)msg->ToObj;

    if ( msg->actId == REDRAW ) {
        myPtr->CDrawFace();
        myPtr->CDrawOutLine();
        m_ShowMouse;
	} else if ( !receiving && msg->mouButton == LCLICK ) {
        receiving       = TRUE;
        if ( !str ) {
            str = new char[256];
            if ( !str )
                perror("Mem pool new");
        }
        myPtr->InitEdit( str );
    } else if (msg->mouButton == LCLICK) {
		myPtr->StopEdit();
    } else switch( msg->actId ) {
		case MOUSE_OFF:
        case CANCEL_CLICK:
			myPtr->StopEdit();
        case KEY_DOWN:
            int result = gStrEditor.CEdit(msg->keyDown);
			myPtr->UpdateBox( result, msg );
            break;
        default:
            break;
    }

    if( myPtr->PrvHandler)
        myPtr->PrvHandler( msg );

	return 0;
}
