#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include "debug.h"
#include "macros.h"
#include "errcode.h"
#include "fileutil.h"
#include "buffer.h"

#ifdef __MSDOS__
#include <dos.h>
#include <dir.h>
#endif

#ifdef __LINUX__
#include <errno.h>
#endif

char *OrigDir = 0, *WorkDir = 0;  // Original & Working working Directory

static int DirIndex = 0;
#define MAX_SAVE_DIRS 5
static char *Dirs[MAX_SAVE_DIRS];

char *PushDir( void )
{
    if ( DirIndex == MAX_SAVE_DIRS ) return 0;
    getcwd( buffer, MAXPATH );
    Dirs[DirIndex] = new char[strlen(buffer)+1];
    strcpy( Dirs[DirIndex], buffer );
    return Dirs[DirIndex++];
}

void _pascal_ Chdir( char *path )
{
    if ( path ) {
        setdisk( toupper(path[0])-'A');
        chdir( path );
    }
}

int PopDir( char *path )
{
    if ( DirIndex ) {
        DirIndex --;
        if ( Dirs[DirIndex] ) {
            if ( path )
                strcpy( path, Dirs[DirIndex]);
            Chdir( Dirs[DirIndex] );
            delete Dirs[DirIndex];
            Dirs[DirIndex] = 0;
            return 1;
        }
    }
    return 0;
}

char * _pascal_ FindString( char c, char *bufPtr, FILE *f, char *str )
{
    int len;
    if ( str )
        len = strlen(str);
    while( f ) {
        if ( fgets(bufPtr, 512, f) == NULL ) break;
        if ( bufPtr[0] == c ) {
            bufPtr[strlen(bufPtr)-1] = 0; // Strip the Return character
            if ( str == 0 )
                return (bufPtr + 1);
            else {
                if ( strncmpi( (bufPtr+1), str, len ) == 0 )
                    return (bufPtr+1);
            }
        }
    }
    return 0;
}

int _pascal_ FetchValues( char c, int *val, FILE *f, int limit, char *str )
{
    int r, i = 0;
    while( f ) {
        if ( fgets(buffer, 512, f) == NULL ) break;
        if ( str[0] == buffer[0] && strcmp( buffer, str ) == 0 )
            break;
        if( buffer[0] == c ) {
            sscanf( buffer+20, "%d", &r );
            val[i++] = r;
            if ( i > limit )
                break;
        }
    }
    return (i >= limit);
}

char * _pascal_ CreateTempFileName( char *name )
{
    static char *NameBuf = 0, *BufChg;
    if ( NameBuf == 0 ) {
        NameBuf = new char[MAXPATH+MAXFILE+MAXEXT+2];
        if ( !NameBuf ) {
            printf("CreateTempName new failed.\n");
            exit( CREATETEMPNAME_NEW_ERROR );
        }
        char *temp = getenv("TEMP");
        if ( !temp ) temp = getenv("TMP");
        if ( temp ) strcpy( NameBuf, temp );
        if ( !temp || !FileExist( NameBuf ) ) {
            if ( getcwd( NameBuf, MAXPATH+MAXFILE+MAXEXT+2 ) == NULL ) {
                printf("CreateTempName getcwd failed %d.\n", errno );
                exit( CREATETEMPNAME_GETCWD_ERROR );
            }
        }
        if ( NameBuf[strlen(NameBuf)-1] != '\\' )
            strcat( NameBuf, "\\" );
        BufChg = NameBuf+(strlen(NameBuf)-1);
    }
    strcpy( BufChg, name );
    return NameBuf;
}

int _pascal_ FileExist( char *name )
{
    FILE *f;
    if ( (f = fopen( name, "r")) == 0 ) return 0;
    fclose( f );
    return 1;
}


//
// Read a file into an unsigned char _FAR_ buffer and return the pointer
// Takes the name of the file
//
char _FAR_* _pascal_ readfile(char *filenamein, int &BufSize, char _FAR_ *buf )
{
    char _FAR_ *ibuff;
	#ifdef __MSDOS__
    unsigned infilesize, nread;
    #endif

    #ifdef __LINUX__
    long infilesize;
    int nread;
    #endif
    
    FILE *fi;

	fi = fopen( filenamein,"rb+" );
    if ( !fi ) {
        fprintf(stderr, "Error opening %s.\n", filenamein);
        return 0;
    }

	fseek ( fi, 0, SEEK_END );
	infilesize = (unsigned) ftell(fi);
	fseek ( fi, 0, SEEK_SET );

    if ( !buf )
        ibuff = new char _FAR_ [infilesize];
    else {
        ibuff = buf;
        if ( BufSize < infilesize ) {
            fclose(fi);
            return 0;
        }
    }

    if ( !ibuff ) return 0;

	#ifdef __MSDOS__
    int fih = fileno( fi );
    _dos_read( fih, ibuff, infilesize, &nread );
    #endif

    #ifdef __LINUX__
    fread( (void*) ibuff, sizeof(char), infilesize, fi );
	#endif

    BufSize = (int)nread;

    fclose(fi);
    return ibuff;
}

//
// Write a char _FAR_ buffer to a file with size bytes
// Takes name of file to be written, pointer to buffer, and size of buffer
//
// returns number of bytes actually written
//
unsigned _pascal_
writefile( char *filenameout, char _FAR_ *obuff, unsigned size )
{
	#ifdef __MSDOS__
    unsigned nwrite;
    #endif
    
	FILE *fo;

    fo = fopen( filenameout,"wb+" );
	if ( !fo ) return 0;

	#ifdef __MSDOS__
	int foh = fileno( fo );
    _dos_write( foh, (void _FAR_*)obuff, size, &nwrite );
    #endif
    
    #ifdef __LINUX__
    size_t nwrite = fwrite( (const void*) obuff, sizeof(char), size, fo );
    #endif

/*	if ferror( fo ) return 0; */

	fclose(fo);
    return nwrite;
}

void _pascal_ InitDirs( char *wd )
{
    char *drv = buffer;
    char *path = buffer + MAXDRIVE;
    char *name = buffer + MAXDRIVE + MAXDIR;
    char *ext = buffer + MAXDRIVE + MAXDIR + MAXFILE;

    // Save Original Drive and path
    OrigDir = PushDir();

    // Get working drive and path( Assume where program is )
    fnsplit( wd, drv, path, name, ext );

    WorkDir = new char[strlen(path)+1+MAXDRIVE];
    strcpy( WorkDir, drv );
    strcat( WorkDir, path );

}

