#include <dos.h>
#include <stdio.h>
#define _DEFAULT_CURSOR_
#include "mousefun.h"

static int Hidden = 1;
int horz_pos, vert_pos;

/*
    Function:       mouse_reset()
	Parameters:
	  (output)      status    status of the mouse
	  (output)      buttons   number of mouse buttons
	Returned:       (function returns nothing)
	Variables:      m1      Local variable for register ax
			m2      Local variable for register bx
	Description:    Resets the mouse and verifies its existence
*/

int mouse_reset ( int *buttons )
{
	int m1, m2;

	_asm {
		xor ax, ax;
		int 33h;
		mov m1, ax;
		mov m2, bx;
	}

    if (buttons) *buttons = m2;
    Hidden = 1;
    mouse_setgcurs(&gcursor_default);
    mouse_show();
    mouse_status();
    return m1;
}

/*-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/

/*
	Function:       mouse_show()
	Parameters:     (none)
	Returned:       (function returns nothing)
	Variables:      (none)
	Description:    Makes the mouse cursor visible
*/
void mouse_show(void)
{
    if ( Hidden ) {
        Hidden = 0;
        _asm {
            mov ax, 1
            int 33h
        }
    }
}

/*-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/

/*
	Function:       mouse_hide()
	parameters:     (none)
	Returned:       (nothing)
	Variables:      (none)
	Description:    makes the mouse cursor invisible
*/

void mouse_hide(void)
{
    if ( !Hidden ) {
        Hidden = 1;
        _asm {
            mov ax, 2
            int 33h
        }
    }
}

/*-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/

/*
    Function:       mouse_status()

    Parameters:
      (output)      left_button     State of the left button
      (output)      right_button    State of the right button
      (output)      horz_pos        Horizontal position of the cursor
      (output)      vert_pos        Vertical position of the cursor

    Returned:       (nothing)

    Variables:      m2      Local Var. for register bx
                    m3      ------------------------cx
                    m4      ------------------------dx

    Description:    Gets the current state of the mouse buttons and
            the mouse cursor position
*/

int mouse_status ( void )

{
    static int m2;

	_asm {
		mov ax, 3
		int 33h
		mov m2, bx
        mov horz_pos, cx
        mov vert_pos, dx
	}
/*
    *left_button = m2 & 1;
    *right_button = ( m2 >> 1 ) & 1;
    horz_pos = m3;
    vert_pos = m4;
*/
    return m2;
}

/*-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/

/*
    Function:       mouse_setpos()
    Parameters
      (input)       horizontal      Horizontal position
      (input)       Vertical        Vertical position
    Returned:       (nothing)
    VAR:            (none)
    Description:    Sets the mouse cursor the the indicated position
*/
void mouse_setpos ( int horizontal, int vertical )
{
	_asm {
		mov ax, 4
		mov cx, horizontal
		mov dx, vertical
		int 33h
	}
}

/*-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/

/*
    Function:       mouse_press()
    Parameters:
      (input)       button      Left or right button
      (output)      status      Status of the button
      (output)      presses     Number of button presses
      (output)      horz_pos    Horizontal position at last press
      (output)      vert_pos    Vertical position at last press
    Returned:       (nothing)

    VAR:    m1      ax         ---\
            m2      bx             |_ Local Var's
            m3      cx             |
            m4      dx         ---/

    Description:    Gets button press information
*/

void mouse_press (  int button, int *status, int *presses,
                    int *horz_pos, int *vert_pos )
{
    int m1, m2, m3, m4;

	_asm {
        mov ax,5
        mov bx, button
        int 33h
        mov m1, ax
        mov m2, bx
        mov m3, cx
        mov m4, dx
    }

    if ( button == LBUTTON )
        *status = m1 & 1;
    else
        *status = ( m1 >> 1 ) & 1;
    *presses = m2;
    *horz_pos = m3;
    *vert_pos = m4;
}

/*-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/

/*
    Function:       mouse_release()

    Parameters:
      (i)           button    Left or right button
      (o)           status    Status of the button
      (o)           releases  Number of button releases
      (o)           horz_pos  Horizontal position at last release
      (o)           vert_pos  Vertical position at last release
    Returned:       (nothing)
    VAR:    m1      ax
        m2      bx
        m3      cx
        m4      dx
    Description:    Gets button release information
*/

void mouse_release( int button, int *status, int *releases,
                    int *horz_pos, int *vert_pos)
{
    int m1, m2, m3, m4;

	_asm {
		mov ax,6
		mov bx, button
		int 33h
		mov m1, ax
		mov m2, bx
		mov m3, cx
		mov m4, dx
	}

    if ( button == LBUTTON )
        *status = m1 & 1;
    else
        *status = ( m1 >> 1 ) & 1;

    *releases = m2;
    *horz_pos = m3;
    *vert_pos = m4;
}

/*-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/

void mouse_sethorz( int horz_min, int horz_max )
{
	_asm {
		mov ax,7
		mov cx, horz_min
		mov dx, horz_max
		int 33h
	}
}

/*-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/

/*
    Function:
    Parameters:
    Returned:       (Function returns nothing)
    Variables:      (none)
    Descriptions:
*/
void mouse_setvert( int vert_min, int vert_max )
{
	_asm {
		mov ax, 8
		mov cx, vert_min
		mov dx, vert_max
		int 33h
	}
}

/*-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/

/*
    Function:
    Parameters:
    Returned:       (Function returns nothing)
    Variables:      (none)
    Descriptions:
*/


void mouse_setgcurs( struct graphics_cursor _FAR_ *cursor )
{
    unsigned cursor_seg = FP_SEG( cursor );
    unsigned cursor_off = FP_OFF( cursor );
    int hotx = cursor->hot_spot_x;
    int hoty = cursor->hot_spot_y;
	_asm {
		mov ax, 9
		mov bx, hotx
		mov cx, hoty
		mov es, cursor_seg
		mov dx, cursor_off
		int 33h
	}
}

/*-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/

/*
    Function:
    Parameters:
    Returned:       (Function returns nothing)
    Variables:      (none)
    Descriptions:
*/


void mouse_settcurs ( int cursor_select, int screen_mask, int cursor_mask)
{
	_asm {
        mov ax, 10
        mov bx, cursor_select
        mov cx, screen_mask
        mov dx, cursor_mask
        int 33h
	}
}

/*-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/

/*
    Function:
    Parameters:
    Returned:       (Function returns nothing)
    Variables:      (none)
    Descriptions:
*/
void mouse_motion( int *horz_mickeys, int *vert_mickeys)
{
    int m3, m4;

	_asm {
		mov ax, 11
		int 33h
		mov m3, cx
		mov m4, dx
	}

    *horz_mickeys = m3;
    *vert_mickeys = m4;
}

/*-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/

/*
    Function:
    Parameters:
    Returned:       (Function returns nothing)
    Variables:      (none)
    Descriptions:
*/
void mouse_setratios( int horizontal, int vertical )
{
	_asm {
		mov ax, 15
		mov cx, horizontal
		mov dx, vertical
		int 33h
	}
}

/*-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/

/*
    Function:
    Parameters:
    Returned:       (Function returns nothing)
    Variables:      (none)
    Descriptions:
*/
void mouse_condoff( int x1, int y1, int x2, int y2)
{
	_asm {
		mov ax, 16
		mov cx, x1
		mov dx, y1
		mov si, x2
		mov di, y2
		int 33h
	}
}

/*-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/

/*
    Function:
    Parameters:
    Returned:       (Function returns nothing)
    Variables:      (none)
    Descriptions:
*/
void mouse_setdouble ( int mickeys_per_second)
{
	_asm {
		mov ax, 19
		mov dx, mickeys_per_second
		int 33h
	}
}

/*-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/

/*
    Function:
    Parameters:
    Returned:       (Function returns nothing)
    Variables:      (none)
    Descriptions:
*/


void mouse_storage ( int *buffer_size)
{
    int m2;

	_asm {
		mov ax, 21
		int 33h
		mov m2, bx
	}

    *buffer_size = m2;
}

/*-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/

/*
    Function:
    Parameters:
    Returned:       (Function returns nothing)
    Variables:      (none)
    Descriptions:
*/

void mouse_save ( char _FAR_ *buffer )
{
    unsigned buffer_seg = FP_SEG ( buffer );
    unsigned buffer_off = FP_OFF ( buffer );

	_asm {
		mov ax, 22
		mov es, buffer_seg
		mov dx, buffer_off
		int 33h
	}
}

/*-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/

/*
    Function:
    Parameters:
    Returned:       (Function returns nothing)
    Variables:      (none)
    Descriptions:
*/


void mouse_restore( char _FAR_ *buffer )
{
    unsigned buffer_seg = FP_SEG ( buffer );
    unsigned buffer_off = FP_OFF ( buffer );

	_asm {
		mov ax, 23
		mov es, buffer_seg
		mov dx, buffer_off
		int 33h
	}
}

/*-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/

/*
    Function:
    Parameters:
    Returned:       (Function returns nothing)
    Variables:      (none)
    Descriptions:
*/

void mouse_setsensitivity(int horz, int vert, int threshold )
{
	_asm {
		mov ax, 26
		mov bx, horz
		mov cx, vert
		mov dx, threshold
		int 33h
	}
}

/*-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/

/*
    Function:
    Parameters:
    Returned:       (Function returns nothing)
    Variables:      (none)
    Descriptions:
*/
void mouse_getsensitivity ( int *horz, int *vert, int *threshold)
{
    int m2, m3, m4;

	_asm {
		mov ax, 27
		int 33h
		mov m2, bx
		mov m3, cx
		mov m4, dx
	}

    *horz = m2;
    *vert = m3;
    *threshold = m4;
}

/*-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/

/*
    Function:
    Parameters:
    Returned:       (Function returns nothing)
    Variables:      (none)
    Descriptions:
*/

void mouse_setmaxrate ( int interrupts_per_second )
{
    int rate;

    if ( interrupts_per_second <= 0 )
        rate = 0;
    else if ( interrupts_per_second > 0 && interrupts_per_second <= 30)
        rate = 1;
    else if ( interrupts_per_second > 30 && interrupts_per_second <= 50 )
        rate = 2;
    else if ( interrupts_per_second > 50 && interrupts_per_second <= 100 )
        rate = 3;
    else
        rate = 4;

	_asm {
		mov ax, 28
		mov bx, rate
		int 33h
	}
}

/*-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/

/*
    Function:
    Parameters:
    Returned:       (Function returns nothing)
    Variables:      (none)
    Descriptions:
*/
void mouse_setpage(int crt_page )
{
	_asm {
		mov ax, 29
		mov bx, crt_page
		int 33h
	}
}

/*-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/

/*
    Function:
    Parameters:
    Returned:       (Function returns nothing)
    Variables:      (none)
    Descriptions:
*/
void mouse_getpage ( int *crt_page )
{
    int m2;

	_asm {
		mov ax, 30
		int 33h
		mov m2, bx
	}

    *crt_page = m2;
}

/*-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/

/*
    Function:
    Parameters:
    Returned:       (Function returns nothing)
    Variables:      (none)
    Descriptions:
*/
void mouse_setlang ( int language )
{
	_asm {
		mov ax, 34
		mov bx, language
		int 33h
	}
}

/*-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/

/*
    Function:
    Parameters:
    Returned:       (Function returns nothing)
    Variables:      (none)
    Descriptions:
*/
void mouse_getlang ( int * language )
{
    int m2;

	_asm {
		mov ax, 35
		int 33h
		mov m2, bx
	}

    *language = m2;
}

/*-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/

/*
    Function:
    Parameters:
    Returned:       (Function returns nothing)
    Variables:      (none)
    Descriptions:
*/


void mouse_getversion ( int *maj, int *min, int *mouse_type, int *irq_num )
{
    int m2, m3;

	_asm {
		mov ax, 36
		int 33h
		mov m2, bx
		mov m3, cx
	}

    *maj = ( m2 >> 12 ) * 10 + (( m2 >> 8 ) & 0xf );
    *min = (( m2 >> 4 ) & 0xf ) * 10 + ( m2 & 0xf );
    *mouse_type = m3 >> 8;
    *irq_num = m3 & 0xff;
}
