/* Mousefun.h
   Header file for mousefun.c */

#ifndef MOUSEFUN_DEFINED

#define LBUTTON 0
#define RBUTTON 1

#define SOFT_TEXT_CURSOR 0
#define HARD_TEXT_CURSOR 1

#define ENGLISH         0
#define FRENCH          1
#define DUTCH           2
#define GERMAN          3
#define SWEDISH         4
#define FINNISH         5
#define SPANISH         6
#define PORTUGUESE      7
#define ITALIAN         8

#define MOUSE_BUS       1
#define MOUSE_SERIAL    2

#define MOUSE_INPORT    3
#define MOUSE_PS2       4
#define MOUSE_HP        5

#define IRQ_PS2         0

/* Structure definition for graphics mode mouse cursors */
struct graphics_cursor
	{
	int screen_mask[16];
	int cursor_mask[16];
	int hot_spot_x;
	int hot_spot_y;
	};

int mouse_reset ( int * );
void mouse_show( void );
void mouse_hide( void );
// void mouse_status( int *, int *, int *, int * );
int mouse_status( void );
void mouse_setpos( int, int );
void mouse_press( int, int *, int *, int *, int * );
void mouse_release ( int, int *, int *, int *, int * );
void mouse_sethorz ( int, int );
void mouse_setvert ( int, int );
void mouse_setgcurs ( struct graphics_cursor _FAR_ * );
void mouse_settcurs( int, int, int );
void mouse_motion ( int *, int * );
void mouse_setratios ( int, int );
void mouse_condoff ( int, int, int, int );
void mouse_setdouble ( int );
void mouse_storage ( int * );
void mouse_save ( char _FAR_ * );
void mouse_restore ( char _FAR_ * );
void mouse_setsensitivity ( int, int, int );
void mouse_getsensitivity( int *, int *, int * );
void mouse_setmaxrate ( int );
void mouse_setpage ( int );
void mouse_getpage ( int * );
void mouse_setlang ( int );
void mouse_getlang ( int * );
void mouse_getversion ( int *,int *, int *, int * );

#define MOUSEFUN_DEFINED
#endif

/*#define _DEFAULT_CURSOR_
#define _HAND_CURSOR_
#define _CHECK_CURSOR
#define _HOUR_CURSOR_
#define _JET_CURSOR_
#define _LEFT_CURSOR_
#define _PLUS_CURSOR_
#define _UP_CURSOR_
#define _X_CURSOR_ */

/* Default graphics mode cursor */
#ifdef _DEFAULT_CURSOR_
static struct graphics_cursor _FAR_ gcursor_default =
	{

	/* screen mask */
	{
	0xCFFF,
	0xC7FF,
	0xC3FF,
	0xC1FF,
	0XC0FF,
	0xC07F,
	0xC03F,
	0xC01F,
	0xC00F,
	0xC007,
	0xC07F,
	0xC43F,
	0xCC3F,
	0xFE1F,
	0xFE1F,
	0xFF1F
	},

	/* CURSOR MASK */
	{
	0x0000,
	0x1000,
	0x1800,
	0x1C00,
	0x1E00,
	0x1F00,
	0x1F80,
	0x1FC0,
	0x1FE0,
	0x1F00,
	0x1B00,
	0x1180,
	0x0180,
	0x00C0,
	0x00C0,
	0x0000
	},
	/* host spot x,y */
	2, 0
	};
#endif

/* Graphic mode cursor, pointing hand */
#ifdef _HAND_CURSOR_
static struct graphics_cursor _FAR_ gcursor_hand =
	{

	/* screen mask */
	{
	0xE1FF,
	0xE1FF,
	0xE1FF,
	0xE1FF,
	0xE1FF,
	0xE000,
	0xE000,
	0x0000,
	0x0000,
	0x0000,
	0x0000,
	0x0000,
	0x0000,
	0x0000,
	0x0000,
	0x0000
	},

	/* cursor mask */
	{
	0x1E00,
	0x1E00,
	0x1E00,
	0x1200,
	0x1200,
	0x13FF,
	0x1249,
	0x1249,
	0xF249,
	0x9001,
	0x9001,
	0x9001,
	0x8001,
	0x8001,
	0x8001,
	0xFFFF
	},

	/* hot spot x,y */
	5, 0
	};
#endif

/* Graphics mode cursor, check mark */
#ifdef _CHECK_CURSOR
static struct graphics_cursor _FAR_ gcursor_check =
	{

	/* screen mask */
	{
	0xFFF0,
	0xFFE0,
	0xFFC0,
	0xFF81,
	0xFF03,
	0x0607,
	0x000F,
	0x001F,
	0xC03F,
	0xF07F,
	0xFFFF,
	0xFFFF,
	0xFFFF,
	0xFFFF,
	0xFFFF,
	0xFFFF
	},

	/* CURSOR MASK */
	{
	0x0000,
	0x0006,
	0x000C,
	0x0018,
	0x0030,
	0x0060,
	0x70C0,
	0x1D80,
	0x0700,
	0x0000,
	0x0000,
	0x0000,
	0x0000,
	0x0000,
	0x0000,
	0x0000
	},

	/* hot spot x,y */
	6, 7
	};
#endif

/* Graphics mode cursor, hour glass */
#ifdef _HOUR_CURSOR_
static struct graphics_cursor _FAR_ gcursor_hour =
	{

	/* screen mask */
	{
	0x0000,
	0X0000,
	0x0000,
	0x0000,
	0x0000,
	0x8001,
	0xC003,
	0xE007,
	0xF00F,
	0xE007,
	0xC003,
	0x8001,
	0x0000,
	0x0000,
	0x0000,
	0x0000
	},

	/* cursor mask */
	{
	0x0000,
	0x7FFE,
	0x6006,
	0x3E3C,
	0x1FF8,
	0x0FF0,
	0x07E0,
	0x03C0,
	0x06E0,
	0x0CB0,
	0x1898,
	0x318C,
	0x63C6,
	0x7FFE,
	0x0000,
	0x0000
	},

	/* hot spot x,y */
	7, 7
	};
#endif

/* Graphics mode cursor, jet aircraft */
#ifdef _JET_CURSOR_
static struct graphics_cursor _FAR_ gcursor_jet =
	{

	/* Screen mask */
	{
	0xFFFF,
	0xFEFF,
	0xFC7F,
	0xF83F,
	0xF83F,
	0xF83F,
	0xF01F,
	0xE00F,
	0xC007,
	0x8003,
	0x8003,
	0xF83F,
	0xF83F,
	0xF01F,
	0xE00F,
	0xFFFF
	},

	/* cursor mask */
	{
	0x0000,
	0x0000,
	0x0100,
	0x0380,
	0x0380,
	0x0380,
	0x07C0,
	0x0FE0,
	0x1FF0,
	0x3FF8,
	0x638C,
	0x0380,
	0x0380,
	0x07C0,
	0x0C60,
	0x0000
	},

	/* hot spot x, y */
	7, 1
	};
#endif

/* Graphics mode cursor, left pointing arrow */
#ifdef _LEFT_CURSOR_
static struct graphics_cursor _FAR_ gcursor_left =
	{

	/* screen mask */
	{
	0xFE1F,
	0xF01F,
	0x0000,
	0x0000,
	0x0000,
	0xF01F,
	0xFE1F,
	0xFFFF,
	0xFFFF,
	0xFFFF,
	0xFFFF,
	0xFFFF,
	0xFFFF,
	0xFFFF,
	0xFFFF,
	0xFFFF
	},

	/* cursor mask */
	{
	0x0000,
	0x00C0,
	0x07C0,
	0x7FFE,
	0x07C0,
	0x00C0,
	0x0000,
	0x0000,
	0x0000,
	0x0000,
	0x0000,
	0x0000,
	0x0000,
	0x0000,
	0x0000,
	0x0000
	},

	/* hot spot x,y */
	0, 3
	};
#endif

/* Graphics mode cursor, plus sign */
#ifdef _PLUS_CURSOR_
static struct graphics_cursor _FAR_ gcursor_plus =
	{

	/* screen mask */
	{
	0xFC3F,
	0xFC3F,
	0xFC3F,
	0x0000,
	0x0000,
	0x0000,
	0xFC3F,
	0xFC3F,
	0xFC3F,
	0xFFFF,
	0xFFFF,
	0xFFFF,
	0xFFFF,
	0xFFFF,
	0xFFFF,
	0xFFFF
	},

	/* cursor mask */
	{
	0x0000,
	0x0180,
	0x0180,
	0x0180,
	0x7FFE,
	0x0180,
	0x0180,
	0x0180,
	0X0000,
	0x0000,
	0x0000,
	0x0000,
	0x0000,
	0x0000,
	0x0000,
	0x0000
	},

	/* hot spot x,y */
	7, 4
	};
#endif

/* Graphics mode cursor, up pointing arrow */
#ifdef _UP_CURSOR_
static struct graphics_cursor _FAR_ gcursor_up =
	{

	/* screen mask */
	{
	0xF9FF,
	0xF0FF,
	0xE07F,
	0xE07F,
	0xC03F,
	0xC03F,
	0x801F,
	0x801F,
	0x000F,
	0x000F,
	0xF0FF,
	0xF0FF,
	0xF0FF,
	0xF0FF,
	0xF0FF,
	0xF0FF
	},

	/* cursor mask */
	{
	0x0000,
	0x0600,
	0x0F00,
	0x0F00,
	0x1F80,
	0x1F80,
	0x3FC0,
	0x3FC0,
	0x7FE0,
	0x0600,
	0x0600,
	0x0600,
	0x0600,
	0x0600,
	0x0600,
	0x0000
	},

	/* hot spot x,y */
	5, 0
	};
#endif

/* Graphics mode cursor, X mark */
#ifdef _X_CURSOR_
static struct graphics_cursor _FAR_ gcursor_x =
	{

	/* screen mask */
	{
	0x07E0,
	0x0180,
	0x0000,
	0xC003,
	0xF00F,
	0xC003,
	0x0000,
	0x0180,
	0x03C0,
	0xFFFF,
	0xFFFF,
	0xFFFF,
	0xFFFF,
	0xFFFF,
	0xFFFF,
	0xFFFF
	},

	/* Cursor mask */
	{
	0x0000,
	0x700E,
	0x1C38,
	0x0660,
	0x03C0,
	0x0660,
	0x1C38,
	0x700E,
	0x0000,
	0x0000,
	0x0000,
	0x0000,
	0x0000,
	0x0000,
	0x0000,
	0x0000
	},

	/* hot spot x, y */
	7, 4
	};
#endif

